from typing import Any, overload
from typing_extensions import Literal

import sqlalchemy.types as sqltypes

from ...engine import default
from ...sql import compiler
from ...sql.elements import Cast
from ...types import (
    BIGINT as BIGINT,
    BINARY as BINARY,
    CHAR as CHAR,
    DATE as DATE,
    DATETIME as DATETIME,
    DECIMAL as DECIMAL,
    FLOAT as FLOAT,
    INTEGER as INTEGER,
    NCHAR as NCHAR,
    NUMERIC as NUMERIC,
    NVARCHAR as NVARCHAR,
    SMALLINT as SMALLINT,
    TEXT as TEXT,
    VARCHAR as VARCHAR,
)
from .json import JSON as JSON

MS_2017_VERSION: Any
MS_2016_VERSION: Any
MS_2014_VERSION: Any
MS_2012_VERSION: Any
MS_2008_VERSION: Any
MS_2005_VERSION: Any
MS_2000_VERSION: Any
RESERVED_WORDS: Any

class REAL(sqltypes.REAL):
    __visit_name__: str
    def __init__(self, **kw) -> None: ...

class TINYINT(sqltypes.Integer):
    __visit_name__: str

class _MSDate(sqltypes.Date):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class TIME(sqltypes.TIME):
    precision: Any
    def __init__(self, precision: Any | None = ..., **kwargs) -> None: ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

_MSTime = TIME

class _BASETIMEIMPL(TIME):
    __visit_name__: str

class _DateTimeBase:
    def bind_processor(self, dialect): ...

class _MSDateTime(_DateTimeBase, sqltypes.DateTime): ...

class SMALLDATETIME(_DateTimeBase, sqltypes.DateTime):
    __visit_name__: str

class DATETIME2(_DateTimeBase, sqltypes.DateTime):
    __visit_name__: str
    precision: Any
    def __init__(self, precision: Any | None = ..., **kw) -> None: ...

class DATETIMEOFFSET(_DateTimeBase, sqltypes.DateTime):
    __visit_name__: str
    precision: Any
    def __init__(self, precision: Any | None = ..., **kw) -> None: ...

class _UnicodeLiteral:
    def literal_processor(self, dialect): ...

class _MSUnicode(_UnicodeLiteral, sqltypes.Unicode): ...
class _MSUnicodeText(_UnicodeLiteral, sqltypes.UnicodeText): ...

class TIMESTAMP(sqltypes._Binary):
    __visit_name__: str
    length: Any
    convert_int: Any
    def __init__(self, convert_int: bool = ...) -> None: ...
    def result_processor(self, dialect, coltype): ...

class ROWVERSION(TIMESTAMP):
    __visit_name__: str

class NTEXT(sqltypes.UnicodeText):
    __visit_name__: str

class VARBINARY(sqltypes.VARBINARY, sqltypes.LargeBinary):
    __visit_name__: str
    filestream: bool
    @overload
    def __init__(self, length: Literal["max"] | None, filestream: Literal[True]) -> None: ...
    @overload
    def __init__(self, *, filestream: Literal[True]) -> None: ...
    @overload
    def __init__(self, length: Any | None = ..., filestream: Literal[False] = ...) -> None: ...

class IMAGE(sqltypes.LargeBinary):
    __visit_name__: str

class XML(sqltypes.Text):
    __visit_name__: str

class BIT(sqltypes.Boolean):
    __visit_name__: str

class MONEY(sqltypes.TypeEngine):
    __visit_name__: str

class SMALLMONEY(sqltypes.TypeEngine):
    __visit_name__: str

class UNIQUEIDENTIFIER(sqltypes.TypeEngine):
    __visit_name__: str

class SQL_VARIANT(sqltypes.TypeEngine):
    __visit_name__: str

class TryCast(Cast):
    __visit_name__: str
    stringify_dialect: str
    inherit_cache: bool
    def __init__(self, *arg, **kw) -> None: ...

try_cast: Any
MSDateTime: Any
MSDate: Any
MSReal = REAL
MSTinyInteger = TINYINT
MSTime = TIME
MSSmallDateTime = SMALLDATETIME
MSDateTime2 = DATETIME2
MSDateTimeOffset = DATETIMEOFFSET
MSText = TEXT
MSNText = NTEXT
MSString = VARCHAR
MSNVarchar = NVARCHAR
MSChar = CHAR
MSNChar = NCHAR
MSBinary = BINARY
MSVarBinary = VARBINARY
MSImage = IMAGE
MSBit = BIT
MSMoney = MONEY
MSSmallMoney = SMALLMONEY
MSUniqueIdentifier = UNIQUEIDENTIFIER
MSVariant = SQL_VARIANT
ischema_names: Any

class MSTypeCompiler(compiler.GenericTypeCompiler):
    def visit_FLOAT(self, type_, **kw): ...
    def visit_TINYINT(self, type_, **kw): ...
    def visit_TIME(self, type_, **kw): ...
    def visit_TIMESTAMP(self, type_, **kw): ...
    def visit_ROWVERSION(self, type_, **kw): ...
    def visit_datetime(self, type_, **kw): ...
    def visit_DATETIMEOFFSET(self, type_, **kw): ...
    def visit_DATETIME2(self, type_, **kw): ...
    def visit_SMALLDATETIME(self, type_, **kw): ...
    def visit_unicode(self, type_, **kw): ...
    def visit_text(self, type_, **kw): ...
    def visit_unicode_text(self, type_, **kw): ...
    def visit_NTEXT(self, type_, **kw): ...
    def visit_TEXT(self, type_, **kw): ...
    def visit_VARCHAR(self, type_, **kw): ...
    def visit_CHAR(self, type_, **kw): ...
    def visit_NCHAR(self, type_, **kw): ...
    def visit_NVARCHAR(self, type_, **kw): ...
    def visit_date(self, type_, **kw): ...
    def visit__BASETIMEIMPL(self, type_, **kw): ...
    def visit_time(self, type_, **kw): ...
    def visit_large_binary(self, type_, **kw): ...
    def visit_IMAGE(self, type_, **kw): ...
    def visit_XML(self, type_, **kw): ...
    def visit_VARBINARY(self, type_, **kw): ...
    def visit_boolean(self, type_, **kw): ...
    def visit_BIT(self, type_, **kw): ...
    def visit_JSON(self, type_, **kw): ...
    def visit_MONEY(self, type_, **kw): ...
    def visit_SMALLMONEY(self, type_, **kw): ...
    def visit_UNIQUEIDENTIFIER(self, type_, **kw): ...
    def visit_SQL_VARIANT(self, type_, **kw): ...

class MSExecutionContext(default.DefaultExecutionContext):
    def pre_exec(self) -> None: ...
    cursor_fetch_strategy: Any
    def post_exec(self) -> None: ...
    def get_lastrowid(self): ...
    @property
    def rowcount(self): ...
    def handle_dbapi_exception(self, e) -> None: ...
    def get_result_cursor_strategy(self, result): ...
    def fire_sequence(self, seq, type_): ...
    def get_insert_default(self, column): ...

class MSSQLCompiler(compiler.SQLCompiler):
    returning_precedes_values: bool
    extract_map: Any
    tablealiases: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def visit_now_func(self, fn, **kw): ...
    def visit_current_date_func(self, fn, **kw): ...
    def visit_length_func(self, fn, **kw): ...
    def visit_char_length_func(self, fn, **kw): ...
    def visit_concat_op_binary(self, binary, operator, **kw): ...
    def visit_true(self, expr, **kw): ...
    def visit_false(self, expr, **kw): ...
    def visit_match_op_binary(self, binary, operator, **kw): ...
    def get_select_precolumns(self, select, **kw): ...
    def get_from_hint_text(self, table, text): ...
    def get_crud_hint_text(self, table, text): ...
    def fetch_clause(self, cs, **kwargs): ...
    def limit_clause(self, cs, **kwargs): ...
    def visit_try_cast(self, element, **kw): ...
    def translate_select_structure(self, select_stmt, **kwargs): ...
    def visit_table(self, table, mssql_aliased: bool = ..., iscrud: bool = ..., **kwargs): ...  # type: ignore[override]
    def visit_alias(self, alias, **kw): ...
    def visit_column(self, column, add_to_result_map: Any | None = ..., **kw): ...  # type: ignore[override]
    def visit_extract(self, extract, **kw): ...
    def visit_savepoint(self, savepoint_stmt): ...
    def visit_rollback_to_savepoint(self, savepoint_stmt): ...
    def visit_binary(self, binary, **kwargs): ...
    def returning_clause(self, stmt, returning_cols): ...
    def get_cte_preamble(self, recursive): ...
    def label_select_column(self, select, column, asfrom): ...
    def for_update_clause(self, select, **kw): ...
    def order_by_clause(self, select, **kw): ...
    def update_from_clause(self, update_stmt, from_table, extra_froms, from_hints, **kw): ...
    def delete_table_clause(self, delete_stmt, from_table, extra_froms): ...
    def delete_extra_from_clause(self, delete_stmt, from_table, extra_froms, from_hints, **kw): ...
    def visit_empty_set_expr(self, type_): ...
    def visit_is_distinct_from_binary(self, binary, operator, **kw): ...
    def visit_is_not_distinct_from_binary(self, binary, operator, **kw): ...
    def visit_json_getitem_op_binary(self, binary, operator, **kw): ...
    def visit_json_path_getitem_op_binary(self, binary, operator, **kw): ...
    def visit_sequence(self, seq, **kw): ...

class MSSQLStrictCompiler(MSSQLCompiler):
    ansi_bind_rules: bool
    def visit_in_op_binary(self, binary, operator, **kw): ...
    def visit_not_in_op_binary(self, binary, operator, **kw): ...
    def render_literal_value(self, value, type_): ...

class MSDDLCompiler(compiler.DDLCompiler):
    def get_column_specification(self, column, **kwargs): ...
    def visit_create_index(self, create, include_schema: bool = ...): ...  # type: ignore[override]
    def visit_drop_index(self, drop): ...
    def visit_primary_key_constraint(self, constraint): ...
    def visit_unique_constraint(self, constraint): ...
    def visit_computed_column(self, generated): ...
    def visit_create_sequence(self, create, **kw): ...
    def visit_identity_column(self, identity, **kw): ...

class MSIdentifierPreparer(compiler.IdentifierPreparer):
    reserved_words: Any
    def __init__(self, dialect) -> None: ...
    def quote_schema(self, schema, force: Any | None = ...): ...

class MSDialect(default.DefaultDialect):
    name: str
    supports_statement_cache: bool
    supports_default_values: bool
    supports_empty_insert: bool
    use_scope_identity: bool
    max_identifier_length: int
    schema_name: str
    implicit_returning: bool
    full_returning: bool
    colspecs: Any
    engine_config_types: Any
    ischema_names: Any
    supports_sequences: bool
    sequences_optional: bool
    default_sequence_base: int
    supports_native_boolean: bool
    non_native_boolean_check_constraint: bool
    supports_unicode_binds: bool
    postfetch_lastrowid: bool
    legacy_schema_aliasing: bool
    server_version_info: Any
    statement_compiler: Any
    ddl_compiler: Any
    type_compiler: Any
    preparer: Any
    construct_arguments: Any
    query_timeout: Any
    deprecate_large_types: Any
    isolation_level: Any
    def __init__(
        self,
        query_timeout: Any | None = ...,
        use_scope_identity: bool = ...,
        schema_name: str = ...,
        isolation_level: Any | None = ...,
        deprecate_large_types: Any | None = ...,
        json_serializer: Any | None = ...,
        json_deserializer: Any | None = ...,
        legacy_schema_aliasing: Any | None = ...,
        ignore_no_transaction_on_rollback: bool = ...,
        **opts,
    ) -> None: ...
    def do_savepoint(self, connection, name) -> None: ...
    def do_release_savepoint(self, connection, name) -> None: ...
    def set_isolation_level(self, connection, level) -> None: ...
    def get_isolation_level(self, dbapi_connection): ...
    def initialize(self, connection) -> None: ...
    def on_connect(self): ...
    def has_table(self, connection, tablename, dbname, owner, schema): ...
    def has_sequence(self, connection, sequencename, dbname, owner, schema): ...
    def get_sequence_names(self, connection, dbname, owner, schema, **kw): ...
    def get_schema_names(self, connection, **kw): ...
    def get_table_names(self, connection, dbname, owner, schema, **kw): ...
    def get_view_names(self, connection, dbname, owner, schema, **kw): ...
    def get_indexes(self, connection, tablename, dbname, owner, schema, **kw): ...
    def get_view_definition(self, connection, viewname, dbname, owner, schema, **kw): ...
    def get_columns(self, connection, tablename, dbname, owner, schema, **kw): ...
    def get_pk_constraint(self, connection, tablename, dbname, owner, schema, **kw): ...
    def get_foreign_keys(self, connection, tablename, dbname, owner, schema, **kw): ...



from logging import Logger
from re import Match, Pattern
from typing import Any
from xml.etree.ElementTree import Element

from markdown import Markdown

from .blockparser import BlockParser

logger: Logger

def build_block_parser(md: Markdown, **kwargs: Any): ...

class BlockProcessor:
    parser: BlockParser
    tab_length: int
    def __init__(self, parser: BlockParser) -> None: ...
    def lastChild(self, parent: Element) -> Element | None: ...
    def detab(self, text: str, length: int | None = ...) -> tuple[str, str]: ...
    def looseDetab(self, text: str, level: int = ...) -> str: ...
    def test(self, parent: Element, block: str) -> bool: ...
    def run(self, parent: Element, blocks: list[str]) -> bool | None: ...

class ListIndentProcessor(BlockProcessor):
    ITEM_TYPES: list[str]
    LIST_TYPES: list[str]
    INDENT_RE: Pattern[str]
    def __init__(self, parser: BlockParser) -> None: ...  # Note: This was done because the args are sent as-is.
    def create_item(self, parent: Element, block: str) -> None: ...
    def get_level(self, parent: Element, block: str) -> tuple[int, Element]: ...

class CodeBlockProcessor(BlockProcessor): ...

class BlockQuoteProcessor(BlockProcessor):
    RE: Pattern[str]
    def clean(self, line: str) -> str: ...

class OListProcessor(BlockProcessor):
    TAG: str = ...
    STARTSWITH: str = ...
    LAZY_OL: bool = ...
    SIBLING_TAGS: list[str]
    RE: Pattern[str]
    CHILD_RE: Pattern[str]
    INDENT_RE: Pattern[str]
    def __init__(self, parser: BlockParser) -> None: ...
    def get_items(self, block: str) -> list[str]: ...

class UListProcessor(OListProcessor):
    TAG: str = ...
    RE: Pattern[str]
    def __init__(self, parser: BlockParser) -> None: ...

class HashHeaderProcessor(BlockProcessor):
    RE: Pattern[str]

class SetextHeaderProcessor(BlockProcessor):
    RE: Pattern[str]

class HRProcessor(BlockProcessor):
    RE: str = ...
    SEARCH_RE: Pattern[str]
    match: Match[str]

class EmptyBlockProcessor(BlockProcessor): ...

class ReferenceProcessor(BlockProcessor):
    RE: Pattern[str]

class ParagraphProcessor(BlockProcessor): ...



from typing import Any, ClassVar

import passlib.utils.handlers as uh

class lmhash(uh.TruncateMixin, uh.HasEncodingContext, uh.StaticHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    truncate_size: ClassVar[int]
    @classmethod
    def raw(cls, secret, encoding: Any | None = ...): ...

class nthash(uh.StaticHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    @classmethod
    def raw(cls, secret): ...
    @classmethod
    def raw_nthash(cls, secret, hex: bool = ...): ...

bsd_nthash: Any

class msdcc(uh.HasUserContext, uh.StaticHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    @classmethod
    def raw(cls, secret, user): ...

class msdcc2(uh.HasUserContext, uh.StaticHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    @classmethod
    def raw(cls, secret, user): ...



from typing import Any

from ...sql.dml import Insert as StandardInsert
from ...sql.elements import ClauseElement
from ...util import memoized_property

class Insert(StandardInsert):
    stringify_dialect: str
    inherit_cache: bool
    @property
    def inserted(self): ...
    @memoized_property
    def inserted_alias(self): ...
    def on_duplicate_key_update(self, *args, **kw) -> None: ...

insert: Any

class OnDuplicateClause(ClauseElement):
    __visit_name__: str
    stringify_dialect: str
    inserted_alias: Any
    update: Any
    def __init__(self, inserted_alias, update) -> None: ...



from typing import Iterator
from typing_extensions import assert_type


class OldStyleIter:
    def __getitem__(self, index: int) -> str:
        return str(index)


for x in iter(OldStyleIter()):
    assert_type(x, str)

assert_type(iter(OldStyleIter()), Iterator[str])
assert_type(next(iter(OldStyleIter())), str)



from re import Pattern
from typing import Any

from . import util

def build_preprocessors(md, **kwargs): ...

class Preprocessor(util.Processor):
    def run(self, lines: list[str]) -> list[str]: ...

class NormalizeWhitespace(Preprocessor): ...

class HtmlBlockPreprocessor(Preprocessor):
    right_tag_patterns: Any
    attrs_pattern: str = ...
    left_tag_pattern: Any
    attrs_re: Any
    left_tag_re: Any
    markdown_in_raw: bool = ...

class ReferencePreprocessor(Preprocessor):
    TITLE: str = ...
    RE: Pattern[str]
    TITLE_RE: Pattern[str]



from collections.abc import Callable, Iterable
from datetime import timedelta
from logging import Logger
from re import Pattern
from typing import Any
from typing_extensions import ParamSpec

_P = ParamSpec("_P")

LOG: Logger

def cross_origin(
    *args: Any,
    origins: str | Pattern[str] | Iterable[str | Pattern[str]] | None = ...,
    methods: str | list[str] | None = ...,
    expose_headers: str | list[str] | None = ...,
    allow_headers: str | Pattern[str] | Iterable[str | Pattern[str]] | None = ...,
    supports_credentials: bool | None = ...,
    max_age: timedelta | int | str | None = ...,
    send_wildcard: bool | None = ...,
    vary_header: bool | None = ...,
    automatic_options: bool | None = ...,
) -> Callable[[Callable[_P, Any]], Callable[_P, Any]]: ...



from _typeshed import Incomplete

from win32com.client import dynamic as dynamic

def GetObject(Pathname: str | None = ..., Class: Incomplete | None = ..., clsctx: Incomplete | None = ...) -> CDispatch: ...
def GetActiveObject(Class, clsctx=...): ...
def Moniker(Pathname, clsctx=...): ...
def Dispatch(
    dispatch,
    userName: Incomplete | None = ...,
    resultCLSID: Incomplete | None = ...,
    typeinfo: Incomplete | None = ...,
    UnicodeToString: Incomplete | None = ...,
    clsctx=...,
): ...
def DispatchEx(
    clsid,
    machine: Incomplete | None = ...,
    userName: Incomplete | None = ...,
    resultCLSID: Incomplete | None = ...,
    typeinfo: Incomplete | None = ...,
    UnicodeToString: Incomplete | None = ...,
    clsctx: Incomplete | None = ...,
): ...

class CDispatch(dynamic.CDispatch):
    def __dir__(self): ...

def CastTo(ob, target, typelib: Incomplete | None = ...): ...

class Constants:
    __dicts__: Incomplete
    def __getattr__(self, a): ...

constants: Incomplete

class EventsProxy:
    def __init__(self, ob) -> None: ...
    def __del__(self) -> None: ...
    def __getattr__(self, attr): ...
    def __setattr__(self, attr, val) -> None: ...

def DispatchWithEvents(clsid, user_event_class): ...
def WithEvents(disp, user_event_class): ...
def getevents(clsid): ...
def Record(name, object): ...

class DispatchBaseClass:
    def __init__(self, oobj: Incomplete | None = ...) -> None: ...
    def __dir__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __getattr__(self, attr): ...
    def __setattr__(self, attr, value) -> None: ...

class CoClassBaseClass:
    def __init__(self, oobj: Incomplete | None = ...) -> None: ...
    def __getattr__(self, attr): ...
    def __setattr__(self, attr, value) -> None: ...
    def __maybe__call__(self, *args, **kwargs): ...
    def __maybe__str__(self, *args): ...
    def __maybe__int__(self, *args): ...
    def __maybe__iter__(self): ...
    def __maybe__len__(self): ...
    def __maybe__nonzero__(self): ...

class VARIANT:
    varianttype: Incomplete
    def __init__(self, vt, value) -> None: ...
    value: Incomplete



from collections.abc import Callable, Container, Iterable, Mapping, Sequence
from typing import Any, NamedTuple
from typing_extensions import TypeAlias

__version__: str

LATEX_ESCAPE_RULES: dict[str, str]
MIN_PADDING: int
PRESERVE_WHITESPACE: bool
WIDE_CHARS_MODE: bool
multiline_formats: dict[str, str]
tabulate_formats: list[str]

class Line(NamedTuple):
    begin: str
    hline: str
    sep: str
    end: str

class DataRow(NamedTuple):
    begin: str
    sep: str
    end: str

_TableFormatLine: TypeAlias = None | Line | Callable[[list[int], list[str]], str]
_TableFormatRow: TypeAlias = None | DataRow | Callable[[list[Any], list[int], list[str]], str]

class TableFormat(NamedTuple):
    lineabove: _TableFormatLine
    linebelowheader: _TableFormatLine
    linebetweenrows: _TableFormatLine
    linebelow: _TableFormatLine
    headerrow: _TableFormatRow
    datarow: _TableFormatRow
    padding: int
    with_header_hide: Container[str] | None

def simple_separated_format(separator: str) -> TableFormat: ...
def tabulate(
    tabular_data: Mapping[str, Iterable[Any]] | Iterable[Iterable[Any]],
    headers: str | dict[str, str] | Sequence[str] = ...,
    tablefmt: str | TableFormat = ...,
    floatfmt: str | Iterable[str] = ...,
    intfmt: str | Iterable[str] = ...,
    numalign: str | None = ...,
    stralign: str | None = ...,
    missingval: str | Iterable[str] = ...,
    showindex: str | bool | Iterable[Any] = ...,
    disable_numparse: bool | Iterable[int] = ...,
    colalign: Iterable[str | None] | None = ...,
    maxcolwidths: int | Iterable[int | None] | None = ...,
    rowalign: str | Iterable[str] | None = ...,
    maxheadercolwidths: int | Iterable[int] | None = ...,
) -> str: ...



import sys
from _typeshed import ReadableBuffer, Self
from collections.abc import Callable, Set as AbstractSet
from typing import Protocol
from typing_extensions import final

if sys.version_info >= (3, 11):
    __all__ = (
        "md5",
        "sha1",
        "sha224",
        "sha256",
        "sha384",
        "sha512",
        "blake2b",
        "blake2s",
        "sha3_224",
        "sha3_256",
        "sha3_384",
        "sha3_512",
        "shake_128",
        "shake_256",
        "new",
        "algorithms_guaranteed",
        "algorithms_available",
        "pbkdf2_hmac",
        "file_digest",
    )
else:
    __all__ = (
        "md5",
        "sha1",
        "sha224",
        "sha256",
        "sha384",
        "sha512",
        "blake2b",
        "blake2s",
        "sha3_224",
        "sha3_256",
        "sha3_384",
        "sha3_512",
        "shake_128",
        "shake_256",
        "new",
        "algorithms_guaranteed",
        "algorithms_available",
        "pbkdf2_hmac",
    )

class _Hash:
    @property
    def digest_size(self) -> int: ...
    @property
    def block_size(self) -> int: ...
    @property
    def name(self) -> str: ...
    def __init__(self, data: ReadableBuffer = ...) -> None: ...
    def copy(self: Self) -> Self: ...
    def digest(self) -> bytes: ...
    def hexdigest(self) -> str: ...
    def update(self, __data: ReadableBuffer) -> None: ...

if sys.version_info >= (3, 9):
    def new(name: str, data: ReadableBuffer = ..., *, usedforsecurity: bool = ...) -> _Hash: ...
    def md5(string: ReadableBuffer = ..., *, usedforsecurity: bool = ...) -> _Hash: ...
    def sha1(string: ReadableBuffer = ..., *, usedforsecurity: bool = ...) -> _Hash: ...
    def sha224(string: ReadableBuffer = ..., *, usedforsecurity: bool = ...) -> _Hash: ...
    def sha256(string: ReadableBuffer = ..., *, usedforsecurity: bool = ...) -> _Hash: ...
    def sha384(string: ReadableBuffer = ..., *, usedforsecurity: bool = ...) -> _Hash: ...
    def sha512(string: ReadableBuffer = ..., *, usedforsecurity: bool = ...) -> _Hash: ...

elif sys.version_info >= (3, 8):
    def new(name: str, data: ReadableBuffer = ...) -> _Hash: ...
    def md5(string: ReadableBuffer = ...) -> _Hash: ...
    def sha1(string: ReadableBuffer = ...) -> _Hash: ...
    def sha224(string: ReadableBuffer = ...) -> _Hash: ...
    def sha256(string: ReadableBuffer = ...) -> _Hash: ...
    def sha384(string: ReadableBuffer = ...) -> _Hash: ...
    def sha512(string: ReadableBuffer = ...) -> _Hash: ...

else:
    def new(name: str, data: ReadableBuffer = ...) -> _Hash: ...
    def md5(__string: ReadableBuffer = ...) -> _Hash: ...
    def sha1(__string: ReadableBuffer = ...) -> _Hash: ...
    def sha224(__string: ReadableBuffer = ...) -> _Hash: ...
    def sha256(__string: ReadableBuffer = ...) -> _Hash: ...
    def sha384(__string: ReadableBuffer = ...) -> _Hash: ...
    def sha512(__string: ReadableBuffer = ...) -> _Hash: ...

algorithms_guaranteed: AbstractSet[str]
algorithms_available: AbstractSet[str]

def pbkdf2_hmac(
    hash_name: str, password: ReadableBuffer, salt: ReadableBuffer, iterations: int, dklen: int | None = ...
) -> bytes: ...

class _VarLenHash:
    digest_size: int
    block_size: int
    name: str
    def __init__(self, data: ReadableBuffer = ...) -> None: ...
    def copy(self) -> _VarLenHash: ...
    def digest(self, __length: int) -> bytes: ...
    def hexdigest(self, __length: int) -> str: ...
    def update(self, __data: ReadableBuffer) -> None: ...

sha3_224 = _Hash
sha3_256 = _Hash
sha3_384 = _Hash
sha3_512 = _Hash
shake_128 = _VarLenHash
shake_256 = _VarLenHash

def scrypt(
    password: ReadableBuffer,
    *,
    salt: ReadableBuffer | None = ...,
    n: int | None = ...,
    r: int | None = ...,
    p: int | None = ...,
    maxmem: int = ...,
    dklen: int = ...,
) -> bytes: ...
@final
class _BlakeHash(_Hash):
    MAX_DIGEST_SIZE: int
    MAX_KEY_SIZE: int
    PERSON_SIZE: int
    SALT_SIZE: int

    if sys.version_info >= (3, 9):
        def __init__(
            self,
            __data: ReadableBuffer = ...,
            *,
            digest_size: int = ...,
            key: ReadableBuffer = ...,
            salt: ReadableBuffer = ...,
            person: ReadableBuffer = ...,
            fanout: int = ...,
            depth: int = ...,
            leaf_size: int = ...,
            node_offset: int = ...,
            node_depth: int = ...,
            inner_size: int = ...,
            last_node: bool = ...,
            usedforsecurity: bool = ...,
        ) -> None: ...
    else:
        def __init__(
            self,
            __data: ReadableBuffer = ...,
            *,
            digest_size: int = ...,
            key: ReadableBuffer = ...,
            salt: ReadableBuffer = ...,
            person: ReadableBuffer = ...,
            fanout: int = ...,
            depth: int = ...,
            leaf_size: int = ...,
            node_offset: int = ...,
            node_depth: int = ...,
            inner_size: int = ...,
            last_node: bool = ...,
        ) -> None: ...

blake2b = _BlakeHash
blake2s = _BlakeHash

if sys.version_info >= (3, 11):
    class _BytesIOLike(Protocol):
        def getbuffer(self) -> ReadableBuffer: ...

    class _FileDigestFileObj(Protocol):
        def readinto(self, __buf: bytearray) -> int: ...
        def readable(self) -> bool: ...

    def file_digest(
        __fileobj: _BytesIOLike | _FileDigestFileObj, __digest: str | Callable[[], _Hash], *, _bufsize: int = ...
    ) -> _Hash: ...



from typing import Any

def skip_if(predicate, reason: Any | None = ...): ...
def fails_if(predicate, reason: Any | None = ...): ...

class compound:
    fails: Any
    skips: Any
    tags: Any
    def __add__(self, other): ...
    def as_skips(self): ...
    def add(self, *others): ...
    def not_(self): ...
    @property
    def enabled(self): ...
    def enabled_for_config(self, config): ...
    def matching_config_reasons(self, config): ...
    def include_test(self, include_tags, exclude_tags): ...
    def __call__(self, fn): ...
    def fail_if(self) -> None: ...

def requires_tag(tagname): ...
def tags(tagnames): ...
def only_if(predicate, reason: Any | None = ...): ...
def succeeds_if(predicate, reason: Any | None = ...): ...

class Predicate:
    @classmethod
    def as_predicate(cls, predicate, description: Any | None = ...): ...

class BooleanPredicate(Predicate):
    value: Any
    description: Any
    def __init__(self, value, description: Any | None = ...) -> None: ...
    def __call__(self, config): ...

class SpecPredicate(Predicate):
    db: Any
    op: Any
    spec: Any
    description: Any
    def __init__(self, db, op: Any | None = ..., spec: Any | None = ..., description: Any | None = ...) -> None: ...
    def __call__(self, config): ...

class LambdaPredicate(Predicate):
    lambda_: Any
    args: Any
    kw: Any
    description: Any
    def __init__(self, lambda_, description: Any | None = ..., args: Any | None = ..., kw: Any | None = ...): ...
    def __call__(self, config): ...

class NotPredicate(Predicate):
    predicate: Any
    description: Any
    def __init__(self, predicate, description: Any | None = ...) -> None: ...
    def __call__(self, config): ...

class OrPredicate(Predicate):
    predicates: Any
    description: Any
    def __init__(self, predicates, description: Any | None = ...) -> None: ...
    def __call__(self, config): ...

def db_spec(*dbs): ...
def open(): ...
def closed(): ...
def fails(reason: Any | None = ...): ...
def future(fn, *arg): ...
def fails_on(db, reason: Any | None = ...): ...
def fails_on_everything_except(*dbs): ...
def skip(db, reason: Any | None = ...): ...
def only_on(dbs, reason: Any | None = ...): ...
def exclude(db, op, spec, reason: Any | None = ...): ...
def against(config, *queries): ...



from _typeshed import Incomplete
from typing import Any
from typing_extensions import TypeAlias

__all__ = ["TqdmCallback"]

_Callback: TypeAlias = Any  # Actually dask.callbacks.Callback

class TqdmCallback(_Callback):
    tqdm_class: type[Incomplete]
    def __init__(
        self, start: Incomplete | None = ..., pretask: Incomplete | None = ..., tqdm_class: type[Incomplete] = ..., **tqdm_kwargs
    ) -> None: ...
    def display(self) -> None: ...



from _typeshed import Self
from collections.abc import Callable, Iterable, Iterator
from re import Pattern
from typing import Any, Generic, TypeVar, overload
from typing_extensions import TypeAlias

from . import BeautifulSoup
from .builder import TreeBuilder
from .formatter import Formatter, _EntitySubstitution

DEFAULT_OUTPUT_ENCODING: str
nonwhitespace_re: Pattern[str]
whitespace_re: Pattern[str]
PYTHON_SPECIFIC_ENCODINGS: set[str]

class NamespacedAttribute(str):
    def __new__(cls: type[Self], prefix: str, name: str | None = ..., namespace: str | None = ...) -> Self: ...

class AttributeValueWithCharsetSubstitution(str): ...

class CharsetMetaAttributeValue(AttributeValueWithCharsetSubstitution):
    def __new__(cls, original_value): ...
    def encode(self, encoding: str) -> str: ...  # type: ignore[override]  # incompatible with str

class ContentMetaAttributeValue(AttributeValueWithCharsetSubstitution):
    CHARSET_RE: Pattern[str]
    def __new__(cls, original_value): ...
    def encode(self, encoding: str) -> str: ...  # type: ignore[override]  # incompatible with str

_PageElementT = TypeVar("_PageElementT", bound=PageElement)
_SimpleStrainable: TypeAlias = str | bool | None | bytes | Pattern[str] | Callable[[str], bool] | Callable[[Tag], bool]
_Strainable: TypeAlias = _SimpleStrainable | Iterable[_SimpleStrainable]
_SimpleNormalizedStrainable: TypeAlias = str | bool | None | Pattern[str] | Callable[[str], bool] | Callable[[Tag], bool]
_NormalizedStrainable: TypeAlias = _SimpleNormalizedStrainable | Iterable[_SimpleNormalizedStrainable]

class PageElement:
    parent: Tag | None
    previous_element: PageElement | None
    next_element: PageElement | None
    next_sibling: PageElement | None
    previous_sibling: PageElement | None
    def setup(
        self,
        parent: Tag | None = ...,
        previous_element: PageElement | None = ...,
        next_element: PageElement | None = ...,
        previous_sibling: PageElement | None = ...,
        next_sibling: PageElement | None = ...,
    ) -> None: ...
    def format_string(self, s: str, formatter: Formatter | str | None) -> str: ...
    def formatter_for_name(self, formatter: Formatter | str | _EntitySubstitution): ...
    nextSibling: PageElement | None
    previousSibling: PageElement | None
    @property
    def stripped_strings(self) -> Iterator[str]: ...
    def get_text(self, separator: str = ..., strip: bool = ..., types: tuple[type[NavigableString], ...] = ...) -> str: ...
    getText = get_text
    @property
    def text(self) -> str: ...
    def replace_with(self: Self, *args: PageElement | str) -> Self: ...
    replaceWith = replace_with
    def unwrap(self: Self) -> Self: ...
    replace_with_children = unwrap
    replaceWithChildren = unwrap
    def wrap(self, wrap_inside: _PageElementT) -> _PageElementT: ...
    def extract(self: Self, _self_index: int | None = ...) -> Self: ...
    def insert(self, position: int, new_child: PageElement | str) -> None: ...
    def append(self, tag: PageElement | str) -> None: ...
    def extend(self, tags: Iterable[PageElement | str]) -> None: ...
    def insert_before(self, *args: PageElement | str) -> None: ...
    def insert_after(self, *args: PageElement | str) -> None: ...
    def find_next(
        self,
        name: _Strainable | SoupStrainer | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        string: _Strainable | None = ...,
        **kwargs: _Strainable,
    ) -> Tag | NavigableString | None: ...
    findNext = find_next
    def find_all_next(
        self,
        name: _Strainable | SoupStrainer | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        string: _Strainable | None = ...,
        limit: int | None = ...,
        **kwargs: _Strainable,
    ) -> ResultSet[PageElement]: ...
    findAllNext = find_all_next
    def find_next_sibling(
        self,
        name: _Strainable | SoupStrainer | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        string: _Strainable | None = ...,
        **kwargs: _Strainable,
    ) -> Tag | NavigableString | None: ...
    findNextSibling = find_next_sibling
    def find_next_siblings(
        self,
        name: _Strainable | SoupStrainer | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        string: _Strainable | None = ...,
        limit: int | None = ...,
        **kwargs: _Strainable,
    ) -> ResultSet[PageElement]: ...
    findNextSiblings = find_next_siblings
    fetchNextSiblings = find_next_siblings
    def find_previous(
        self,
        name: _Strainable | SoupStrainer | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        string: _Strainable | None = ...,
        **kwargs: _Strainable,
    ) -> Tag | NavigableString | None: ...
    findPrevious = find_previous
    def find_all_previous(
        self,
        name: _Strainable | SoupStrainer | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        string: _Strainable | None = ...,
        limit: int | None = ...,
        **kwargs: _Strainable,
    ) -> ResultSet[PageElement]: ...
    findAllPrevious = find_all_previous
    fetchPrevious = find_all_previous
    def find_previous_sibling(
        self,
        name: _Strainable | SoupStrainer | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        string: _Strainable | None = ...,
        **kwargs: _Strainable,
    ) -> Tag | NavigableString | None: ...
    findPreviousSibling = find_previous_sibling
    def find_previous_siblings(
        self,
        name: _Strainable | SoupStrainer | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        string: _Strainable | None = ...,
        limit: int | None = ...,
        **kwargs: _Strainable,
    ) -> ResultSet[PageElement]: ...
    findPreviousSiblings = find_previous_siblings
    fetchPreviousSiblings = find_previous_siblings
    def find_parent(
        self,
        name: _Strainable | SoupStrainer | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        **kwargs: _Strainable,
    ) -> Tag | None: ...
    findParent = find_parent
    def find_parents(
        self,
        name: _Strainable | SoupStrainer | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        limit: int | None = ...,
        **kwargs: _Strainable,
    ) -> ResultSet[Tag]: ...
    findParents = find_parents
    fetchParents = find_parents
    @property
    def next(self) -> Tag | NavigableString | None: ...
    @property
    def previous(self) -> Tag | NavigableString | None: ...
    @property
    def next_elements(self) -> Iterable[PageElement]: ...
    @property
    def next_siblings(self) -> Iterable[PageElement]: ...
    @property
    def previous_elements(self) -> Iterable[PageElement]: ...
    @property
    def previous_siblings(self) -> Iterable[PageElement]: ...
    @property
    def parents(self) -> Iterable[Tag]: ...
    @property
    def decomposed(self) -> bool: ...
    def nextGenerator(self) -> Iterable[PageElement]: ...
    def nextSiblingGenerator(self) -> Iterable[PageElement]: ...
    def previousGenerator(self) -> Iterable[PageElement]: ...
    def previousSiblingGenerator(self) -> Iterable[PageElement]: ...
    def parentGenerator(self) -> Iterable[Tag]: ...

class NavigableString(str, PageElement):
    PREFIX: str
    SUFFIX: str
    known_xml: bool | None
    def __new__(cls: type[Self], value: str | bytes) -> Self: ...
    def __copy__(self: Self) -> Self: ...
    def __getnewargs__(self) -> tuple[str]: ...
    def output_ready(self, formatter: Formatter | str | None = ...) -> str: ...
    @property
    def name(self) -> None: ...
    @property
    def strings(self) -> Iterable[str]: ...

class PreformattedString(NavigableString):
    PREFIX: str
    SUFFIX: str
    def output_ready(self, formatter: Formatter | str | None = ...) -> str: ...

class CData(PreformattedString):
    PREFIX: str
    SUFFIX: str

class ProcessingInstruction(PreformattedString):
    PREFIX: str
    SUFFIX: str

class XMLProcessingInstruction(ProcessingInstruction):
    PREFIX: str
    SUFFIX: str

class Comment(PreformattedString):
    PREFIX: str
    SUFFIX: str

class Declaration(PreformattedString):
    PREFIX: str
    SUFFIX: str

class Doctype(PreformattedString):
    @classmethod
    def for_name_and_ids(cls, name: str | None, pub_id: str, system_id: str) -> Doctype: ...
    PREFIX: str
    SUFFIX: str

class Stylesheet(NavigableString): ...
class Script(NavigableString): ...
class TemplateString(NavigableString): ...

class Tag(PageElement):
    parser_class: type[BeautifulSoup] | None
    name: str
    namespace: str | None
    prefix: str | None
    sourceline: int | None
    sourcepos: int | None
    known_xml: bool | None
    attrs: dict[str, str]
    contents: list[PageElement]
    hidden: bool
    can_be_empty_element: bool | None
    cdata_list_attributes: list[str] | None
    preserve_whitespace_tags: list[str] | None
    def __init__(
        self,
        parser: BeautifulSoup | None = ...,
        builder: TreeBuilder | None = ...,
        name: str | None = ...,
        namespace: str | None = ...,
        prefix: str | None = ...,
        attrs: dict[str, str] | None = ...,
        parent: Tag | None = ...,
        previous: PageElement | None = ...,
        is_xml: bool | None = ...,
        sourceline: int | None = ...,
        sourcepos: int | None = ...,
        can_be_empty_element: bool | None = ...,
        cdata_list_attributes: list[str] | None = ...,
        preserve_whitespace_tags: list[str] | None = ...,
        interesting_string_types: type[NavigableString] | tuple[type[NavigableString], ...] | None = ...,
        namespaces: dict[str, str] | None = ...,
    ) -> None: ...
    parserClass: type[BeautifulSoup] | None
    def __copy__(self: Self) -> Self: ...
    @property
    def is_empty_element(self) -> bool: ...
    @property
    def isSelfClosing(self) -> bool: ...
    @property
    def string(self) -> str | None: ...
    @string.setter
    def string(self, string: str) -> None: ...
    DEFAULT_INTERESTING_STRING_TYPES: tuple[type[NavigableString], ...]
    @property
    def strings(self) -> Iterable[str]: ...
    def decompose(self) -> None: ...
    def clear(self, decompose: bool = ...) -> None: ...
    def smooth(self) -> None: ...
    def index(self, element: PageElement) -> int: ...
    def get(self, key: str, default: str | list[str] | None = ...) -> str | list[str] | None: ...
    def get_attribute_list(self, key: str, default: str | list[str] | None = ...) -> list[str]: ...
    def has_attr(self, key: str) -> bool: ...
    def __hash__(self) -> int: ...
    def __getitem__(self, key: str) -> str | list[str]: ...
    def __iter__(self) -> Iterator[PageElement]: ...
    def __len__(self) -> int: ...
    def __contains__(self, x: object) -> bool: ...
    def __bool__(self) -> bool: ...
    def __setitem__(self, key: str, value: str | list[str]) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def __getattr__(self, tag: str) -> Tag | None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __unicode__(self) -> str: ...
    def encode(
        self, encoding: str = ..., indent_level: int | None = ..., formatter: str | Formatter = ..., errors: str = ...
    ) -> bytes: ...
    def decode(self, indent_level: int | None = ..., eventual_encoding: str = ..., formatter: str | Formatter = ...) -> str: ...
    @overload
    def prettify(self, encoding: str, formatter: str | Formatter = ...) -> bytes: ...
    @overload
    def prettify(self, encoding: None = ..., formatter: str | Formatter = ...) -> str: ...
    def decode_contents(
        self, indent_level: int | None = ..., eventual_encoding: str = ..., formatter: str | Formatter = ...
    ) -> str: ...
    def encode_contents(self, indent_level: int | None = ..., encoding: str = ..., formatter: str | Formatter = ...) -> bytes: ...
    def renderContents(self, encoding: str = ..., prettyPrint: bool = ..., indentLevel: int = ...) -> bytes: ...
    def find(
        self,
        name: _Strainable | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        recursive: bool = ...,
        string: _Strainable | None = ...,
        **kwargs: _Strainable,
    ) -> Tag | NavigableString | None: ...
    findChild = find
    def find_all(
        self,
        name: _Strainable | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        recursive: bool = ...,
        string: _Strainable | None = ...,
        limit: int | None = ...,
        **kwargs: _Strainable,
    ) -> ResultSet[Any]: ...
    __call__ = find_all
    findAll = find_all
    findChildren = find_all
    @property
    def children(self) -> Iterable[PageElement]: ...
    @property
    def descendants(self) -> Iterable[PageElement]: ...
    def select_one(
        self, selector: str, namespaces: Any | None = ..., *, flags: int = ..., custom: dict[str, str] | None = ...
    ) -> Tag | None: ...
    def select(
        self,
        selector: str,
        namespaces: Any | None = ...,
        limit: int | None = ...,
        *,
        flags: int = ...,
        custom: dict[str, str] | None = ...,
    ) -> ResultSet[Tag]: ...
    def childGenerator(self) -> Iterable[PageElement]: ...
    def recursiveChildGenerator(self) -> Iterable[PageElement]: ...
    def has_key(self, key: str) -> bool: ...

class SoupStrainer:
    name: _NormalizedStrainable
    attrs: dict[str, _NormalizedStrainable]
    string: _NormalizedStrainable
    def __init__(
        self,
        name: _Strainable | None = ...,
        attrs: dict[str, _Strainable] | _Strainable = ...,
        string: _Strainable | None = ...,
        **kwargs: _Strainable,
    ) -> None: ...
    def search_tag(self, markup_name: Tag | str | None = ..., markup_attrs=...): ...
    searchTag = search_tag
    def search(self, markup: PageElement | Iterable[PageElement]): ...

class ResultSet(list[_PageElementT], Generic[_PageElementT]):
    source: SoupStrainer
    @overload
    def __init__(self, source: SoupStrainer) -> None: ...
    @overload
    def __init__(self, source: SoupStrainer, result: Iterable[_PageElementT]) -> None: ...



from collections.abc import Mapping
from typing import Any

from boto.connection import AWSQueryConnection

class KMSConnection(AWSQueryConnection):
    APIVersion: str
    DefaultRegionName: str
    DefaultRegionEndpoint: str
    ServiceName: str
    TargetPrefix: str
    ResponseError: type[Exception]
    region: Any
    def __init__(self, **kwargs) -> None: ...
    def create_alias(self, alias_name: str, target_key_id: str) -> dict[str, Any] | None: ...
    def create_grant(
        self,
        key_id: str,
        grantee_principal: str,
        retiring_principal: str | None = ...,
        operations: list[str] | None = ...,
        constraints: dict[str, dict[str, str]] | None = ...,
        grant_tokens: list[str] | None = ...,
    ) -> dict[str, Any] | None: ...
    def create_key(
        self, policy: str | None = ..., description: str | None = ..., key_usage: str | None = ...
    ) -> dict[str, Any] | None: ...
    def decrypt(
        self, ciphertext_blob: bytes, encryption_context: Mapping[str, Any] | None = ..., grant_tokens: list[str] | None = ...
    ) -> dict[str, Any] | None: ...
    def delete_alias(self, alias_name: str) -> dict[str, Any] | None: ...
    def describe_key(self, key_id: str) -> dict[str, Any] | None: ...
    def disable_key(self, key_id: str) -> dict[str, Any] | None: ...
    def disable_key_rotation(self, key_id: str) -> dict[str, Any] | None: ...
    def enable_key(self, key_id: str) -> dict[str, Any] | None: ...
    def enable_key_rotation(self, key_id: str) -> dict[str, Any] | None: ...
    def encrypt(
        self,
        key_id: str,
        plaintext: bytes,
        encryption_context: Mapping[str, Any] | None = ...,
        grant_tokens: list[str] | None = ...,
    ) -> dict[str, Any] | None: ...
    def generate_data_key(
        self,
        key_id: str,
        encryption_context: Mapping[str, Any] | None = ...,
        number_of_bytes: int | None = ...,
        key_spec: str | None = ...,
        grant_tokens: list[str] | None = ...,
    ) -> dict[str, Any] | None: ...
    def generate_data_key_without_plaintext(
        self,
        key_id: str,
        encryption_context: Mapping[str, Any] | None = ...,
        key_spec: str | None = ...,
        number_of_bytes: int | None = ...,
        grant_tokens: list[str] | None = ...,
    ) -> dict[str, Any] | None: ...
    def generate_random(self, number_of_bytes: int | None = ...) -> dict[str, Any] | None: ...
    def get_key_policy(self, key_id: str, policy_name: str) -> dict[str, Any] | None: ...
    def get_key_rotation_status(self, key_id: str) -> dict[str, Any] | None: ...
    def list_aliases(self, limit: int | None = ..., marker: str | None = ...) -> dict[str, Any] | None: ...
    def list_grants(self, key_id: str, limit: int | None = ..., marker: str | None = ...) -> dict[str, Any] | None: ...
    def list_key_policies(self, key_id: str, limit: int | None = ..., marker: str | None = ...) -> dict[str, Any] | None: ...
    def list_keys(self, limit: int | None = ..., marker: str | None = ...) -> dict[str, Any] | None: ...
    def put_key_policy(self, key_id: str, policy_name: str, policy: str) -> dict[str, Any] | None: ...
    def re_encrypt(
        self,
        ciphertext_blob: bytes,
        destination_key_id: str,
        source_encryption_context: Mapping[str, Any] | None = ...,
        destination_encryption_context: Mapping[str, Any] | None = ...,
        grant_tokens: list[str] | None = ...,
    ) -> dict[str, Any] | None: ...
    def retire_grant(self, grant_token: str) -> dict[str, Any] | None: ...
    def revoke_grant(self, key_id: str, grant_id: str) -> dict[str, Any] | None: ...
    def update_key_description(self, key_id: str, description: str) -> dict[str, Any] | None: ...



from ctypes import Array, Structure, _Pointer, c_uint
from ctypes.wintypes import BOOL, DWORD, HMONITOR, INT, LARGE_INTEGER, LONG, PFLOAT, POINT, RECT, UINT, ULARGE_INTEGER, WCHAR
from typing_extensions import TypedDict

from d3dshot.dll import _HRESULT, _IUnknown, _ProcessFunc, _ProcessFuncRegionArg, _ProcessFuncReturn
from d3dshot.dll.d3d import ID3D11Device

class _DXGIOutputPosition(TypedDict):
    left: LONG
    top: LONG
    right: LONG
    bottom: LONG

class _DXGIOutput(TypedDict):
    name: str
    position: _DXGIOutputPosition
    resolution: tuple[tuple[LONG, LONG], tuple[LONG, LONG]]
    rotation: int
    is_attached_to_desktop: bool

class LUID(Structure):
    LowPart: DWORD
    HighPart: LONG

class DXGI_ADAPTER_DESC1(Structure):
    Description: Array[WCHAR]
    VendorId: UINT
    DeviceId: UINT
    SubSysId: UINT
    Revision: UINT
    DedicatedVideoMemory: ULARGE_INTEGER
    DedicatedSystemMemory: ULARGE_INTEGER
    SharedSystemMemory: ULARGE_INTEGER
    AdapterLuid: LUID
    Flags: UINT

class DXGI_OUTPUT_DESC(Structure):
    DeviceName: Array[WCHAR]
    DesktopCoordinates: RECT
    AttachedToDesktop: BOOL
    Rotation: UINT
    Monitor: HMONITOR

class DXGI_OUTDUPL_POINTER_POSITION(Structure):
    Position: POINT
    Visible: BOOL

class DXGI_OUTDUPL_FRAME_INFO(Structure):
    LastPresentTime: LARGE_INTEGER
    LastMouseUpdateTime: LARGE_INTEGER
    AccumulatedFrames: UINT
    RectsCoalesced: BOOL
    ProtectedContentMaskedOut: BOOL
    PointerPosition: DXGI_OUTDUPL_POINTER_POSITION
    TotalMetadataBufferSize: UINT
    PointerShapeBufferSize: UINT

class DXGI_MAPPED_RECT(Structure):
    Pitch: INT
    pBits: PFLOAT

class IDXGIObject(_IUnknown):
    def SetPrivateData(self) -> _HRESULT: ...
    def SetPrivateDataInterface(self) -> _HRESULT: ...
    def GetPrivateData(self) -> _HRESULT: ...
    def GetParent(self) -> _HRESULT: ...

class IDXGIDeviceSubObject(IDXGIObject):
    def GetDevice(self) -> _HRESULT: ...

class IDXGIResource(IDXGIDeviceSubObject):
    def GetSharedHandle(self) -> _HRESULT: ...
    def GetUsage(self) -> _HRESULT: ...
    def SetEvictionPriority(self) -> _HRESULT: ...
    def GetEvictionPriority(self) -> _HRESULT: ...

class IDXGISurface(IDXGIDeviceSubObject):
    def GetDesc(self) -> _HRESULT: ...
    def Map(self, __pLockedRect: _Pointer[DXGI_MAPPED_RECT], __MapFlags: UINT) -> _HRESULT: ...
    def Unmap(self) -> _HRESULT: ...

class IDXGIOutputDuplication(IDXGIObject):
    def GetDesc(self) -> None: ...
    def AcquireNextFrame(
        self,
        __TimeoutInMilliseconds: UINT,
        __pFrameInfo: _Pointer[DXGI_OUTDUPL_FRAME_INFO],
        __ppDesktopResource: _Pointer[_Pointer[IDXGIResource]],
    ) -> _HRESULT: ...
    def GetFrameDirtyRects(self) -> _HRESULT: ...
    def GetFrameMoveRects(self) -> _HRESULT: ...
    def GetFramePointerShape(self) -> _HRESULT: ...
    def MapDesktopSurface(self) -> _HRESULT: ...
    def UnMapDesktopSurface(self) -> _HRESULT: ...
    def ReleaseFrame(self) -> _HRESULT: ...

class IDXGIOutput(IDXGIObject):
    def GetDesc(self, __pDesc: _Pointer[DXGI_OUTPUT_DESC]) -> _HRESULT: ...
    def GetDisplayModeList(self) -> _HRESULT: ...
    def FindClosestMatchingMode(self) -> _HRESULT: ...
    def WaitForVBlank(self) -> _HRESULT: ...
    def TakeOwnership(self) -> _HRESULT: ...
    def ReleaseOwnership(self) -> None: ...
    def GetGammaControlCapabilities(self) -> _HRESULT: ...
    def SetGammaControl(self) -> _HRESULT: ...
    def GetGammaControl(self) -> _HRESULT: ...
    def SetDisplaySurface(self) -> _HRESULT: ...
    def GetDisplaySurfaceData(self) -> _HRESULT: ...
    def GetFrameStatistics(self) -> _HRESULT: ...

class IDXGIOutput1(IDXGIOutput):
    def GetDisplayModeList1(self) -> _HRESULT: ...
    def FindClosestMatchingMode1(self) -> _HRESULT: ...
    def GetDisplaySurfaceData1(self) -> _HRESULT: ...
    def DuplicateOutput(
        self, __pDevice: _Pointer[ID3D11Device], __ppOutputDuplication: _Pointer[_Pointer[IDXGIOutputDuplication]]
    ) -> _HRESULT: ...

class IDXGIAdapter(IDXGIObject):
    def EnumOutputs(self, __Output: UINT, __ppOutput: _Pointer[_Pointer[IDXGIOutput]]) -> _HRESULT: ...
    def GetDesc(self) -> _HRESULT: ...
    def CheckInterfaceSupport(self) -> _HRESULT: ...

class IDXGIAdapter1(IDXGIAdapter):
    def GetDesc1(self, __pDesc: _Pointer[DXGI_ADAPTER_DESC1]) -> _HRESULT: ...

class IDXGIFactory(IDXGIObject):
    def EnumAdapters(self) -> _HRESULT: ...
    def MakeWindowAssociation(self) -> _HRESULT: ...
    def GetWindowAssociation(self) -> _HRESULT: ...
    def CreateSwapChain(self) -> _HRESULT: ...
    def CreateSoftwareAdapter(self) -> _HRESULT: ...

class IDXGIFactory1(IDXGIFactory):
    def EnumAdapters1(self, __Adapter: c_uint, __ppAdapter: _Pointer[_Pointer[IDXGIAdapter1]]) -> _HRESULT: ...
    def IsCurrent(self) -> BOOL: ...

def initialize_dxgi_factory() -> _Pointer[IDXGIFactory1]: ...
def discover_dxgi_adapters(dxgi_factory: IDXGIFactory1) -> list[_Pointer[IDXGIAdapter1]]: ...
def describe_dxgi_adapter(dxgi_adapter: IDXGIAdapter1) -> Array[WCHAR]: ...
def discover_dxgi_outputs(dxgi_adapter: IDXGIAdapter) -> list[_Pointer[IDXGIOutput1]]: ...
def describe_dxgi_output(dxgi_output: IDXGIOutput) -> _DXGIOutput: ...
def initialize_dxgi_output_duplication(
    dxgi_output: IDXGIOutput1, d3d_device: _Pointer[ID3D11Device]
) -> _Pointer[IDXGIOutputDuplication]: ...
def get_dxgi_output_duplication_frame(
    dxgi_output_duplication: IDXGIOutputDuplication,
    d3d_device: ID3D11Device,
    process_func: _ProcessFunc[_ProcessFuncRegionArg, _ProcessFuncReturn] | None = ...,
    width: int = ...,
    height: int = ...,
    region: _ProcessFuncRegionArg = ...,
    rotation: int = ...,
) -> _ProcessFuncReturn | None: ...



from typing import Any

from passlib.exc import (
    InvalidTokenError as InvalidTokenError,
    MalformedTokenError as MalformedTokenError,
    TokenError as TokenError,
    UsedTokenError as UsedTokenError,
)
from passlib.utils import SequenceMixin

class AppWallet:
    salt_size: int
    encrypt_cost: int
    default_tag: Any
    def __init__(
        self,
        secrets: Any | None = ...,
        default_tag: Any | None = ...,
        encrypt_cost: Any | None = ...,
        secrets_path: Any | None = ...,
    ) -> None: ...
    @property
    def has_secrets(self): ...
    def get_secret(self, tag): ...
    def encrypt_key(self, key): ...
    def decrypt_key(self, enckey): ...

class TOTP:
    min_json_version: int
    json_version: int
    wallet: Any
    now: Any
    digits: int
    alg: str
    label: Any
    issuer: Any
    period: int
    changed: bool
    @classmethod
    def using(
        cls,
        digits: Any | None = ...,
        alg: Any | None = ...,
        period: Any | None = ...,
        issuer: Any | None = ...,
        wallet: Any | None = ...,
        now: Any | None = ...,
        **kwds,
    ): ...
    @classmethod
    def new(cls, **kwds): ...
    def __init__(
        self,
        key: Any | None = ...,
        format: str = ...,
        new: bool = ...,
        digits: Any | None = ...,
        alg: Any | None = ...,
        size: Any | None = ...,
        period: Any | None = ...,
        label: Any | None = ...,
        issuer: Any | None = ...,
        changed: bool = ...,
        **kwds,
    ) -> None: ...
    @property
    def key(self): ...
    @key.setter
    def key(self, value) -> None: ...
    @property
    def encrypted_key(self): ...
    @encrypted_key.setter
    def encrypted_key(self, value) -> None: ...
    @property
    def hex_key(self): ...
    @property
    def base32_key(self): ...
    def pretty_key(self, format: str = ..., sep: str = ...): ...
    @classmethod
    def normalize_time(cls, time): ...
    def normalize_token(self_or_cls, token): ...
    def generate(self, time: Any | None = ...): ...
    @classmethod
    def verify(cls, token, source, **kwds): ...
    def match(self, token, time: Any | None = ..., window: int = ..., skew: int = ..., last_counter: Any | None = ...): ...
    @classmethod
    def from_source(cls, source): ...
    @classmethod
    def from_uri(cls, uri): ...
    def to_uri(self, label: Any | None = ..., issuer: Any | None = ...): ...
    @classmethod
    def from_json(cls, source): ...
    def to_json(self, encrypt: Any | None = ...): ...
    @classmethod
    def from_dict(cls, source): ...
    def to_dict(self, encrypt: Any | None = ...): ...

class TotpToken(SequenceMixin):
    totp: Any
    token: Any
    counter: Any
    def __init__(self, totp, token, counter) -> None: ...
    @property
    def start_time(self): ...
    @property
    def expire_time(self): ...
    @property
    def remaining(self): ...
    @property
    def valid(self): ...

class TotpMatch(SequenceMixin):
    totp: Any
    counter: int
    time: int
    window: int
    def __init__(self, totp, counter, time, window: int = ...) -> None: ...
    @property
    def expected_counter(self): ...
    @property
    def skipped(self): ...
    @property
    def expire_time(self): ...
    @property
    def cache_seconds(self): ...
    @property
    def cache_time(self): ...



class UnsupportedFormatException(Exception): ...
class InvalidCarrierException(Exception): ...
class SpanContextCorruptedException(Exception): ...

class Format:
    BINARY: str
    TEXT_MAP: str
    HTTP_HEADERS: str



from ctypes import _Pointer
from typing_extensions import TypedDict

from d3dshot.dll import _ProcessFunc, _ProcessFuncRegionArg, _ProcessFuncReturn
from d3dshot.dll.d3d import ID3D11Device, ID3D11DeviceContext
from d3dshot.dll.dxgi import IDXGIAdapter, IDXGIOutput1, IDXGIOutputDuplication

class _PositionDict(TypedDict):
    left: int
    top: int
    right: int
    bottom: int

class Display:
    name: str
    adapter_name: str
    resolution: tuple[int, int]
    position: _PositionDict
    rotation: int
    scale_factor: float
    is_primary: bool
    hmonitor: int
    dxgi_output: IDXGIOutput1 | None
    dxgi_adapter: _Pointer[IDXGIAdapter] | None
    # Note that Display.d3d_device and Display.d3d_device_context can never be None.
    # Despite initially being set to None in __init__,
    # they're always immediately set in _initialize_dxgi_output_duplication()
    d3d_device: ID3D11Device
    d3d_device_context: ID3D11DeviceContext
    dxgi_output_duplication: _Pointer[IDXGIOutputDuplication]

    def __init__(
        self,
        name: str | None = ...,
        adapter_name: str | None = ...,
        resolution: tuple[int, int] | None = ...,
        position: _PositionDict | None = ...,
        rotation: int | None = ...,
        scale_factor: float | None = ...,
        is_primary: bool = ...,
        hmonitor: int | None = ...,
        dxgi_output: IDXGIOutput1 | None = ...,
        dxgi_adapter: _Pointer[IDXGIAdapter] | None = ...,
    ) -> None: ...
    def capture(
        self, process_func: _ProcessFunc[_ProcessFuncRegionArg, _ProcessFuncReturn] | None, region: _ProcessFuncRegionArg = ...
    ) -> _ProcessFuncReturn: ...
    @classmethod
    def discover_displays(cls) -> list[Display]: ...



from _typeshed import Incomplete

from pythoncom import (
    DISPID_COLLECT as DISPID_COLLECT,
    DISPID_CONSTRUCTOR as DISPID_CONSTRUCTOR,
    DISPID_DESTRUCTOR as DISPID_DESTRUCTOR,
    DISPID_UNKNOWN as DISPID_UNKNOWN,
)
from win32com.server.dispatcher import DispatcherTrace as DispatcherTrace, DispatcherWin32trace as DispatcherWin32trace
from win32com.server.exception import COMException as COMException

S_OK: int
IDispatchType: Incomplete
IUnknownType: Incomplete
error: Incomplete
regSpec: str
regPolicy: str
regDispatcher: str
regAddnPath: str

def CreateInstance(clsid, reqIID): ...

class BasicWrapPolicy:
    def __init__(self, object) -> None: ...

class MappedWrapPolicy(BasicWrapPolicy): ...
class DesignatedWrapPolicy(MappedWrapPolicy): ...
class EventHandlerPolicy(DesignatedWrapPolicy): ...
class DynamicPolicy(BasicWrapPolicy): ...

DefaultPolicy = DesignatedWrapPolicy

def resolve_func(spec): ...
def call_func(spec, *args): ...

DISPATCH_METHOD: int
DISPATCH_PROPERTYGET: int
DISPATCH_PROPERTYPUT: int
DISPATCH_PROPERTYPUTREF: int
DISPID_EVALUATE: int
DISPID_NEWENUM: int
DISPID_PROPERTYPUT: int
DISPID_STARTENUM: int
DISPID_VALUE: int



def populate(source, source_mapper, dest, dest_mapper, synchronize_pairs, uowcommit, flag_cascaded_pks) -> None: ...
def bulk_populate_inherit_keys(source_dict, source_mapper, synchronize_pairs) -> None: ...
def clear(dest, dest_mapper, synchronize_pairs) -> None: ...
def update(source, source_mapper, dest, old_prefix, synchronize_pairs) -> None: ...
def populate_dict(source, source_mapper, dict_, synchronize_pairs) -> None: ...
def source_modified(uowcommit, source, source_mapper, synchronize_pairs): ...



from typing import Any

class _CommonFile:
    encoding: Any
    return_unicode: Any
    autosave: bool
    @classmethod
    def from_string(cls, data, **kwds): ...
    @classmethod
    def from_path(cls, path, **kwds): ...
    def __init__(
        self,
        path: Any | None = ...,
        new: bool = ...,
        autoload: bool = ...,
        autosave: bool = ...,
        encoding: str = ...,
        return_unicode=...,
    ) -> None: ...
    @property
    def path(self): ...
    @path.setter
    def path(self, value) -> None: ...
    @property
    def mtime(self): ...
    def load_if_changed(self): ...
    def load(self, path: Any | None = ..., force: bool = ...): ...
    def load_string(self, data) -> None: ...
    def save(self, path: Any | None = ...) -> None: ...
    def to_string(self): ...

class HtpasswdFile(_CommonFile):
    context: Any
    def __init__(self, path: Any | None = ..., default_scheme: Any | None = ..., context=..., **kwds) -> None: ...
    def users(self): ...
    def set_password(self, user, password): ...
    def update(self, user, password): ...
    def get_hash(self, user): ...
    def set_hash(self, user, hash): ...
    def find(self, user): ...
    def delete(self, user): ...
    def check_password(self, user, password): ...
    def verify(self, user, password): ...

class HtdigestFile(_CommonFile):
    default_realm: Any
    def __init__(self, path: Any | None = ..., default_realm: Any | None = ..., **kwds) -> None: ...
    def realms(self): ...
    def users(self, realm: Any | None = ...): ...
    def set_password(self, user, realm: Any | None = ..., password=...): ...
    def update(self, user, realm, password): ...
    def get_hash(self, user, realm: Any | None = ...): ...
    def set_hash(self, user, realm: Any | None = ..., hash=...): ...
    def find(self, user, realm): ...
    def delete(self, user, realm: Any | None = ...): ...
    def delete_realm(self, realm): ...
    def check_password(self, user, realm: Any | None = ..., password=...): ...
    def verify(self, user, realm, password): ...



import datetime
from collections.abc import Iterable
from decimal import Decimal
from typing_extensions import TypedDict

from .feedback import _Feedback
from .matching import _Match
from .time_estimates import _TimeEstimate

class _Result(_TimeEstimate, TypedDict):
    password: str
    guesses: Decimal
    guesses_log10: float
    sequence: list[_Match]
    calc_time: datetime.timedelta
    feedback: _Feedback

def zxcvbn(password: str, user_inputs: Iterable[object] | None = ...) -> _Result: ...



from _typeshed import Self
from os import stat_result

class SFTPAttributes:
    FLAG_SIZE: int
    FLAG_UIDGID: int
    FLAG_PERMISSIONS: int
    FLAG_AMTIME: int
    FLAG_EXTENDED: int
    st_size: int | None
    st_uid: int | None
    st_gid: int | None
    st_mode: int | None
    st_atime: int | None
    st_mtime: int | None
    filename: str  # only when from_stat() is used
    longname: str  # only when from_stat() is used
    attr: dict[str, str]
    def __init__(self) -> None: ...
    @classmethod
    def from_stat(cls: type[Self], obj: stat_result, filename: str | None = ...) -> Self: ...
    def asbytes(self) -> bytes: ...



from collections import deque
from collections.abc import Mapping, Sequence
from logging import Logger
from threading import Condition, Lock
from typing import Any

from .channel import HTTPChannel
from .utilities import Error

rename_headers: Mapping[str, str]
hop_by_hop: frozenset[Any]

class ThreadedTaskDispatcher:
    stop_count: int = ...
    active_count: int = ...
    logger: Logger = ...
    queue_logger: Logger = ...
    threads: set[Any] = ...
    queue: deque[Task] = ...
    lock: Lock = ...
    queue_cv: Condition = ...
    thread_exit_cv: Condition = ...
    def start_new_thread(self, target: Any, args: Any) -> None: ...
    def handler_thread(self, thread_no: int) -> None: ...
    def set_thread_count(self, count: int) -> None: ...
    def add_task(self, task: Task) -> None: ...
    def shutdown(self, cancel_pending: bool = ..., timeout: int = ...) -> bool: ...

class Task:
    close_on_finish: bool = ...
    status: str = ...
    wrote_header: bool = ...
    start_time: int = ...
    content_length: int | None = ...
    content_bytes_written: int = ...
    logged_write_excess: bool = ...
    logged_write_no_body: bool = ...
    complete: bool = ...
    chunked_response: bool = ...
    logger: Logger = ...
    channel: HTTPChannel = ...
    request: Error = ...
    response_headers: Sequence[tuple[str, str]] = ...
    version: str = ...
    def __init__(self, channel: HTTPChannel, request: Error) -> None: ...
    def service(self) -> None: ...
    @property
    def has_body(self) -> bool: ...
    def build_response_header(self) -> bytes: ...
    def remove_content_length_header(self) -> None: ...
    def start(self) -> None: ...
    def finish(self) -> None: ...
    def write(self, data: bytes) -> None: ...

class ErrorTask(Task):
    complete: bool = ...
    status: str = ...
    close_on_finish: bool = ...
    content_length: int = ...
    def execute(self) -> None: ...

class WSGITask(Task):
    environ: Any | None = ...
    response_headers: Sequence[tuple[str, str]] = ...
    complete: bool = ...
    status: str = ...
    content_length: int = ...
    close_on_finish: bool = ...
    def execute(self) -> None: ...
    def get_environment(self) -> Any: ...



import sys
from _typeshed import Incomplete
from collections.abc import Callable
from ctypes import _CData, c_ulong
from ctypes.wintypes import PFLOAT
from typing import TypeVar
from typing_extensions import TypeAlias

from d3dshot.capture_output import _Frame

_ProcessFuncRegionArg = TypeVar("_ProcessFuncRegionArg", tuple[int, int, int, int], None)
_ProcessFuncReturn = TypeVar("_ProcessFuncReturn", _Frame, None)
# The _ProcessFunc alias is used in multiple submodules
_ProcessFunc: TypeAlias = Callable[[PFLOAT, int, int, int, int, _ProcessFuncRegionArg, int], _ProcessFuncReturn]  # noqa: Y047

if sys.platform == "win32":
    from ctypes import HRESULT

    _HRESULT: TypeAlias = HRESULT
else:
    _HRESULT: TypeAlias = Incomplete

# TODO: Use comtypes.IUnknown once we can import non-types dependencies
# See: #5768
class _IUnknown(_CData):
    def QueryInterface(self, interface: type, iid: _CData | None = ...) -> _HRESULT: ...
    def AddRef(self) -> c_ulong: ...
    def Release(self) -> c_ulong: ...



from collections.abc import Iterable, Sequence
from typing import NoReturn

from ..collection import Collection
from ..parser import Parser, ParserContext, ParseResult

def complete(
    names: Iterable[str], core: ParseResult, initial_context: ParserContext, collection: Collection, parser: Parser
) -> NoReturn: ...
def print_task_names(collection: Collection) -> None: ...
def print_completion_script(shell: str, names: Sequence[str]) -> None: ...



import datetime
from collections.abc import Iterable
from typing import Any
from typing_extensions import TypeAlias

from ._common import weekday as weekdaybase

YEARLY: int
MONTHLY: int
WEEKLY: int
DAILY: int
HOURLY: int
MINUTELY: int
SECONDLY: int

class weekday(weekdaybase): ...

weekdays: tuple[weekday, weekday, weekday, weekday, weekday, weekday, weekday]
MO: weekday
TU: weekday
WE: weekday
TH: weekday
FR: weekday
SA: weekday
SU: weekday

class rrulebase:
    def __init__(self, cache: bool = ...) -> None: ...
    def __iter__(self): ...
    def __getitem__(self, item): ...
    def __contains__(self, item): ...
    def count(self): ...
    def before(self, dt, inc: bool = ...): ...
    def after(self, dt, inc: bool = ...): ...
    def xafter(self, dt, count: Any | None = ..., inc: bool = ...): ...
    def between(self, after, before, inc: bool = ..., count: int = ...): ...

class rrule(rrulebase):
    def __init__(
        self,
        freq,
        dtstart: datetime.date | None = ...,
        interval: int = ...,
        wkst: weekday | int | None = ...,
        count: int | None = ...,
        until: datetime.date | int | None = ...,
        bysetpos: int | Iterable[int] | None = ...,
        bymonth: int | Iterable[int] | None = ...,
        bymonthday: int | Iterable[int] | None = ...,
        byyearday: int | Iterable[int] | None = ...,
        byeaster: int | Iterable[int] | None = ...,
        byweekno: int | Iterable[int] | None = ...,
        byweekday: int | weekday | Iterable[int] | Iterable[weekday] | None = ...,
        byhour: int | Iterable[int] | None = ...,
        byminute: int | Iterable[int] | None = ...,
        bysecond: int | Iterable[int] | None = ...,
        cache: bool = ...,
    ) -> None: ...
    def replace(self, **kwargs): ...

class _iterinfo:
    rrule: Any = ...
    def __init__(self, rrule) -> None: ...
    yearlen: int = ...
    nextyearlen: int = ...
    yearordinal: int = ...
    yearweekday: int = ...
    mmask: Any = ...
    mdaymask: Any = ...
    nmdaymask: Any = ...
    wdaymask: Any = ...
    mrange: Any = ...
    wnomask: Any = ...
    nwdaymask: Any = ...
    eastermask: Any = ...
    lastyear: int = ...
    lastmonth: int = ...
    def rebuild(self, year, month): ...
    def ydayset(self, year, month, day): ...
    def mdayset(self, year, month, day): ...
    def wdayset(self, year, month, day): ...
    def ddayset(self, year, month, day): ...
    def htimeset(self, hour, minute, second): ...
    def mtimeset(self, hour, minute, second): ...
    def stimeset(self, hour, minute, second): ...

_RRule: TypeAlias = rrule

class rruleset(rrulebase):
    class _genitem:
        dt: Any = ...
        genlist: Any = ...
        gen: Any = ...
        def __init__(self, genlist, gen) -> None: ...
        def __next__(self): ...
        next: Any = ...
        def __lt__(self, other): ...
        def __gt__(self, other): ...
        def __eq__(self, other): ...
        def __ne__(self, other): ...

    def __init__(self, cache: bool = ...) -> None: ...
    def rrule(self, rrule: _RRule): ...
    def rdate(self, rdate): ...
    def exrule(self, exrule): ...
    def exdate(self, exdate): ...

class _rrulestr:
    def __call__(self, s, **kwargs) -> rrule | rruleset: ...

rrulestr: _rrulestr



from typing_extensions import Literal

from stripe import Event, error as error

class Webhook:
    DEFAULT_TOLERANCE: int
    @staticmethod
    def construct_event(
        payload: bytes | str, sig_header: str, secret: str, tolerance: int = ..., api_key: str | None = ...
    ) -> Event: ...

class WebhookSignature:
    EXPECTED_SCHEME: str
    @classmethod
    def verify_header(cls, payload: bytes | str, header: str, secret: str, tolerance: int | None = ...) -> Literal[True]: ...
    @staticmethod
    def _compute_signature(payload: str, secret: str) -> str: ...



from typing import Any

from .. import util
from ..util import HasMemoized
from .visitors import ExtendedInternalTraversal, InternalTraversal

SKIP_TRAVERSE: Any
COMPARE_FAILED: bool
COMPARE_SUCCEEDED: bool
NO_CACHE: Any
CACHE_IN_PLACE: Any
CALL_GEN_CACHE_KEY: Any
STATIC_CACHE_KEY: Any
PROPAGATE_ATTRS: Any
ANON_NAME: Any

def compare(obj1, obj2, **kw): ...

class HasCacheKey:
    inherit_cache: Any

class MemoizedHasCacheKey(HasCacheKey, HasMemoized): ...

class CacheKey:
    def __hash__(self): ...
    def to_offline_string(self, statement_cache, statement, parameters): ...
    def __eq__(self, other): ...

class _CacheKey(ExtendedInternalTraversal):
    visit_has_cache_key: Any
    visit_clauseelement: Any
    visit_clauseelement_list: Any
    visit_annotations_key: Any
    visit_clauseelement_tuple: Any
    visit_memoized_select_entities: Any
    visit_string: Any
    visit_boolean: Any
    visit_operator: Any
    visit_plain_obj: Any
    visit_statement_hint_list: Any
    visit_type: Any
    visit_anon_name: Any
    visit_propagate_attrs: Any
    def visit_with_context_options(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_inspectable(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_string_list(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_multi(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_multi_list(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_has_cache_key_tuples(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_has_cache_key_list(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_executable_options(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_inspectable_list(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_clauseelement_tuples(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_fromclause_ordered_set(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_clauseelement_unordered_set(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_named_ddl_element(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_prefix_sequence(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_setup_join_tuple(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_table_hint_list(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_plain_dict(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_dialect_options(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_string_clauseelement_dict(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_string_multi_dict(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_fromclause_canonical_column_collection(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_unknown_structure(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_dml_ordered_values(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_dml_values(self, attrname, obj, parent, anon_map, bindparams): ...
    def visit_dml_multi_values(self, attrname, obj, parent, anon_map, bindparams): ...

class HasCopyInternals: ...

class _CopyInternals(InternalTraversal):
    def visit_clauseelement(self, attrname, parent, element, clone=..., **kw): ...
    def visit_clauseelement_list(self, attrname, parent, element, clone=..., **kw): ...
    def visit_clauseelement_tuple(self, attrname, parent, element, clone=..., **kw): ...
    def visit_executable_options(self, attrname, parent, element, clone=..., **kw): ...
    def visit_clauseelement_unordered_set(self, attrname, parent, element, clone=..., **kw): ...
    def visit_clauseelement_tuples(self, attrname, parent, element, clone=..., **kw): ...
    def visit_string_clauseelement_dict(self, attrname, parent, element, clone=..., **kw): ...
    def visit_setup_join_tuple(self, attrname, parent, element, clone=..., **kw): ...
    def visit_memoized_select_entities(self, attrname, parent, element, **kw): ...
    def visit_dml_ordered_values(self, attrname, parent, element, clone=..., **kw): ...
    def visit_dml_values(self, attrname, parent, element, clone=..., **kw): ...
    def visit_dml_multi_values(self, attrname, parent, element, clone=..., **kw): ...
    def visit_propagate_attrs(self, attrname, parent, element, clone=..., **kw): ...

class _GetChildren(InternalTraversal):
    def visit_has_cache_key(self, element, **kw): ...
    def visit_clauseelement(self, element, **kw): ...
    def visit_clauseelement_list(self, element, **kw): ...
    def visit_clauseelement_tuple(self, element, **kw): ...
    def visit_clauseelement_tuples(self, element, **kw): ...
    def visit_fromclause_canonical_column_collection(self, element, **kw): ...
    def visit_string_clauseelement_dict(self, element, **kw): ...
    def visit_fromclause_ordered_set(self, element, **kw): ...
    def visit_clauseelement_unordered_set(self, element, **kw): ...
    def visit_setup_join_tuple(self, element, **kw) -> None: ...
    def visit_memoized_select_entities(self, element, **kw): ...
    def visit_dml_ordered_values(self, element, **kw) -> None: ...
    def visit_dml_values(self, element, **kw) -> None: ...
    def visit_dml_multi_values(self, element, **kw): ...
    def visit_propagate_attrs(self, element, **kw): ...

class anon_map(dict[Any, Any]):
    index: int
    def __init__(self) -> None: ...
    def __missing__(self, key): ...

class TraversalComparatorStrategy(InternalTraversal, util.MemoizedSlots):
    stack: Any
    cache: Any
    anon_map: Any
    def __init__(self) -> None: ...
    def compare(self, obj1, obj2, **kw): ...
    def compare_inner(self, obj1, obj2, **kw): ...
    def visit_has_cache_key(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_propagate_attrs(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_has_cache_key_list(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_executable_options(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_clauseelement(self, attrname, left_parent, left, right_parent, right, **kw) -> None: ...
    def visit_fromclause_canonical_column_collection(self, attrname, left_parent, left, right_parent, right, **kw) -> None: ...
    def visit_fromclause_derived_column_collection(self, attrname, left_parent, left, right_parent, right, **kw) -> None: ...
    def visit_string_clauseelement_dict(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_clauseelement_tuples(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_clauseelement_list(self, attrname, left_parent, left, right_parent, right, **kw) -> None: ...
    def visit_clauseelement_tuple(self, attrname, left_parent, left, right_parent, right, **kw) -> None: ...
    def visit_clauseelement_unordered_set(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_fromclause_ordered_set(self, attrname, left_parent, left, right_parent, right, **kw) -> None: ...
    def visit_string(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_string_list(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_anon_name(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_boolean(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_operator(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_type(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_plain_dict(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_dialect_options(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_annotations_key(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_with_context_options(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_plain_obj(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_named_ddl_element(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_prefix_sequence(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_setup_join_tuple(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_memoized_select_entities(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_table_hint_list(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_statement_hint_list(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_unknown_structure(self, attrname, left_parent, left, right_parent, right, **kw) -> None: ...
    def visit_dml_ordered_values(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_dml_values(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def visit_dml_multi_values(self, attrname, left_parent, left, right_parent, right, **kw): ...
    def compare_clauselist(self, left, right, **kw): ...
    def compare_binary(self, left, right, **kw): ...
    def compare_bindparam(self, left, right, **kw): ...

class ColIdentityComparatorStrategy(TraversalComparatorStrategy):
    def compare_column_element(self, left, right, use_proxies: bool = ..., equivalents=..., **kw): ...
    def compare_column(self, left, right, **kw): ...
    def compare_label(self, left, right, **kw): ...
    def compare_table(self, left, right, **kw): ...



from collections.abc import Iterable
from decimal import Decimal
from re import Pattern
from typing_extensions import TypedDict

from .adjacency_graphs import _Graph
from .matching import _Match

def calc_average_degree(graph: _Graph) -> float: ...

BRUTEFORCE_CARDINALITY: int
MIN_GUESSES_BEFORE_GROWING_SEQUENCE: int
MIN_SUBMATCH_GUESSES_SINGLE_CHAR: int
MIN_SUBMATCH_GUESSES_MULTI_CHAR: int
MIN_YEAR_SPACE: int
REFERENCE_YEAR: int

class _GuessesResult(TypedDict):
    password: str
    guesses: int
    guesses_log10: float
    sequence: list[_Match]

def nCk(n: int, k: int) -> float: ...
def most_guessable_match_sequence(password: str, matches: Iterable[_Match], _exclude_additive: bool = ...) -> _GuessesResult: ...
def estimate_guesses(match: _Match, password: str) -> Decimal: ...
def bruteforce_guesses(match: _Match) -> int: ...
def dictionary_guesses(match: _Match) -> int: ...
def repeat_guesses(match: _Match) -> Decimal: ...
def sequence_guesses(match: _Match) -> int: ...
def regex_guesses(match: _Match) -> int | None: ...
def date_guesses(match: _Match) -> int: ...

KEYBOARD_AVERAGE_DEGREE: float
KEYPAD_AVERAGE_DEGREE: float
KEYBOARD_STARTING_POSITIONS: int
KEYPAD_STARTING_POSITIONS: int

def spatial_guesses(match: _Match) -> int: ...

START_UPPER: Pattern[str]
END_UPPER: Pattern[str]
ALL_UPPER: Pattern[str]
ALL_LOWER: Pattern[str]

def uppercase_variations(match: _Match) -> int: ...
def l33t_variations(match: _Match) -> int: ...



from .exceptions.exceptions import InvalidDaemonAddressException as InvalidDaemonAddressException

DAEMON_ADDRESS_KEY: str
DEFAULT_ADDRESS: str

class DaemonConfig:
    def __init__(self, daemon_address=...) -> None: ...
    @property
    def udp_ip(self): ...
    @property
    def udp_port(self): ...
    @property
    def tcp_ip(self): ...
    @property
    def tcp_port(self): ...



from abc import ABCMeta, abstractmethod

from cryptography.hazmat.backends.interfaces import DSABackend
from cryptography.hazmat.primitives.asymmetric import AsymmetricVerificationContext
from cryptography.hazmat.primitives.asymmetric.utils import Prehashed
from cryptography.hazmat.primitives.hashes import HashAlgorithm
from cryptography.hazmat.primitives.serialization import Encoding, KeySerializationEncryption, PrivateFormat, PublicFormat

class DSAParameters(metaclass=ABCMeta):
    @abstractmethod
    def generate_private_key(self) -> DSAPrivateKey: ...

class DSAParametersWithNumbers(DSAParameters):
    @abstractmethod
    def parameter_numbers(self) -> DSAParameterNumbers: ...

class DSAParameterNumbers:
    @property
    def p(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def g(self) -> int: ...
    def __init__(self, p: int, q: int, g: int) -> None: ...
    def parameters(self, backend: DSABackend | None = ...) -> DSAParameters: ...

class DSAPrivateKey(metaclass=ABCMeta):
    @property
    @abstractmethod
    def key_size(self) -> int: ...
    @abstractmethod
    def parameters(self) -> DSAParameters: ...
    @abstractmethod
    def public_key(self) -> DSAPublicKey: ...
    @abstractmethod
    def sign(self, data: bytes, algorithm: HashAlgorithm | Prehashed) -> bytes: ...

class DSAPrivateKeyWithSerialization(DSAPrivateKey):
    @abstractmethod
    def private_bytes(
        self, encoding: Encoding, format: PrivateFormat, encryption_algorithm: KeySerializationEncryption
    ) -> bytes: ...
    @abstractmethod
    def private_numbers(self) -> DSAPrivateNumbers: ...

class DSAPrivateNumbers:
    @property
    def x(self) -> int: ...
    @property
    def public_numbers(self) -> DSAPublicNumbers: ...
    def __init__(self, x: int, public_numbers: DSAPublicNumbers) -> None: ...

class DSAPublicKey(metaclass=ABCMeta):
    @property
    @abstractmethod
    def key_size(self) -> int: ...
    @abstractmethod
    def public_bytes(self, encoding: Encoding, format: PublicFormat) -> bytes: ...
    @abstractmethod
    def public_numbers(self) -> DSAPublicNumbers: ...
    @abstractmethod
    def verifier(self, signature: bytes, signature_algorithm: HashAlgorithm | Prehashed) -> AsymmetricVerificationContext: ...
    @abstractmethod
    def verify(self, signature: bytes, data: bytes, algorithm: HashAlgorithm | Prehashed) -> None: ...

DSAPublicKeyWithSerialization = DSAPublicKey

class DSAPublicNumbers:
    @property
    def y(self) -> int: ...
    @property
    def parameter_numbers(self) -> DSAParameterNumbers: ...
    def __init__(self, y: int, parameter_numbers: DSAParameterNumbers) -> None: ...

def generate_parameters(key_size: int, backend: DSABackend | None = ...) -> DSAParameters: ...
def generate_private_key(key_size: int, backend: DSABackend | None = ...) -> DSAPrivateKey: ...



import abc
from datetime import datetime, timedelta, tzinfo
from typing import ClassVar

def tzname_in_python2(namefunc): ...
def enfold(dt: datetime, fold: int = ...): ...

class _DatetimeWithFold(datetime):
    @property
    def fold(self): ...

# Doesn't actually have ABCMeta as the metaclass at runtime,
# but mypy complains if we don't have it in the stub.
# See discussion in #8908
class _tzinfo(tzinfo, metaclass=abc.ABCMeta):
    def is_ambiguous(self, dt: datetime) -> bool: ...
    def fromutc(self, dt: datetime) -> datetime: ...

class tzrangebase(_tzinfo):
    def __init__(self) -> None: ...
    def utcoffset(self, dt: datetime | None) -> timedelta | None: ...
    def dst(self, dt: datetime | None) -> timedelta | None: ...
    def tzname(self, dt: datetime | None) -> str: ...
    def fromutc(self, dt: datetime) -> datetime: ...
    def is_ambiguous(self, dt: datetime) -> bool: ...
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def __ne__(self, other): ...
    __reduce__ = object.__reduce__



from collections.abc import Callable
from typing_extensions import Literal, TypedDict

_DEFAULT_DELIMITER: str

class _EmojiListReturn(TypedDict):
    emoji: str
    match_start: int
    match_end: int

def emojize(
    string: str,
    delimiters: tuple[str, str] = ...,
    variant: Literal["text_type", "emoji_type", None] = ...,
    language: str = ...,
    version: float | None = ...,
    handle_version: str | Callable[[str, dict[str, str]], str] | None = ...,
) -> str: ...
def demojize(
    string: str,
    delimiters: tuple[str, str] = ...,
    language: str = ...,
    version: float | None = ...,
    handle_version: str | Callable[[str, dict[str, str]], str] | None = ...,
) -> str: ...
def replace_emoji(string: str, replace: str | Callable[[str, dict[str, str]], str] = ..., version: float = ...) -> str: ...
def emoji_list(string: str) -> list[_EmojiListReturn]: ...
def distinct_emoji_list(string: str) -> list[str]: ...
def emoji_count(string: str, unique: bool = ...) -> int: ...
def version(string: str) -> float: ...
def is_emoji(string: str) -> bool: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class CustomProperty(Serialisable):
    tagname: str
    name: Any
    def __init__(self, name: Any | None = ...) -> None: ...

class CustomProperties(Serialisable):
    tagname: str
    customPr: Any
    __elements__: Any
    def __init__(self, customPr=...) -> None: ...



from _typeshed import Incomplete
from html.parser import HTMLParser
from typing import Any

FRAME: int
ALL: int
NONE: int
HEADER: int
DEFAULT: int
MSWORD_FRIENDLY: int
PLAIN_COLUMNS: int
MARKDOWN: int
ORGMODE: int
RANDOM: int
SINGLE_BORDER: int
DOUBLE_BORDER: int
BASE_ALIGN_VALUE: str

class PrettyTable:
    encoding: Any
    def __init__(self, field_names: Any | None = ..., **kwargs): ...
    def __getattr__(self, name): ...
    def __getitem__(self, index): ...
    @property
    def field_names(self): ...
    @field_names.setter
    def field_names(self, val) -> None: ...
    @property
    def align(self): ...
    @align.setter
    def align(self, val) -> None: ...
    @property
    def valign(self): ...
    @valign.setter
    def valign(self, val) -> None: ...
    @property
    def max_width(self): ...
    @max_width.setter
    def max_width(self, val) -> None: ...
    @property
    def min_width(self): ...
    @min_width.setter
    def min_width(self, val) -> None: ...
    @property
    def min_table_width(self): ...
    @min_table_width.setter
    def min_table_width(self, val) -> None: ...
    @property
    def max_table_width(self): ...
    @max_table_width.setter
    def max_table_width(self, val) -> None: ...
    @property
    def fields(self): ...
    @fields.setter
    def fields(self, val) -> None: ...
    @property
    def title(self): ...
    @title.setter
    def title(self, val) -> None: ...
    @property
    def start(self): ...
    @start.setter
    def start(self, val) -> None: ...
    @property
    def end(self): ...
    @end.setter
    def end(self, val) -> None: ...
    @property
    def sortby(self): ...
    @sortby.setter
    def sortby(self, val) -> None: ...
    @property
    def reversesort(self): ...
    @reversesort.setter
    def reversesort(self, val) -> None: ...
    @property
    def sort_key(self): ...
    @sort_key.setter
    def sort_key(self, val) -> None: ...
    @property
    def header(self): ...
    @header.setter
    def header(self, val) -> None: ...
    @property
    def header_style(self): ...
    @header_style.setter
    def header_style(self, val) -> None: ...
    @property
    def border(self): ...
    @border.setter
    def border(self, val) -> None: ...
    @property
    def hrules(self): ...
    @hrules.setter
    def hrules(self, val) -> None: ...
    @property
    def vrules(self): ...
    @vrules.setter
    def vrules(self, val) -> None: ...
    @property
    def int_format(self): ...
    @int_format.setter
    def int_format(self, val) -> None: ...
    @property
    def float_format(self): ...
    @float_format.setter
    def float_format(self, val) -> None: ...
    @property
    def padding_width(self): ...
    @padding_width.setter
    def padding_width(self, val) -> None: ...
    @property
    def left_padding_width(self): ...
    @left_padding_width.setter
    def left_padding_width(self, val) -> None: ...
    @property
    def right_padding_width(self): ...
    @right_padding_width.setter
    def right_padding_width(self, val) -> None: ...
    @property
    def vertical_char(self): ...
    @vertical_char.setter
    def vertical_char(self, val) -> None: ...
    @property
    def horizontal_char(self): ...
    @horizontal_char.setter
    def horizontal_char(self, val) -> None: ...
    @property
    def junction_char(self): ...
    @junction_char.setter
    def junction_char(self, val) -> None: ...
    @property
    def format(self): ...
    @format.setter
    def format(self, val) -> None: ...
    @property
    def print_empty(self): ...
    @print_empty.setter
    def print_empty(self, val) -> None: ...
    @property
    def attributes(self): ...
    @attributes.setter
    def attributes(self, val) -> None: ...
    @property
    def oldsortslice(self): ...
    @oldsortslice.setter
    def oldsortslice(self, val) -> None: ...
    @property
    def bottom_junction_char(self): ...
    @bottom_junction_char.setter
    def bottom_junction_char(self, val) -> None: ...
    @property
    def bottom_left_junction_char(self): ...
    @bottom_left_junction_char.setter
    def bottom_left_junction_char(self, val) -> None: ...
    @property
    def bottom_right_junction_char(self): ...
    @bottom_right_junction_char.setter
    def bottom_right_junction_char(self, val) -> None: ...
    @property
    def custom_format(self): ...
    @custom_format.setter
    def custom_format(self, val) -> None: ...
    @property
    def horizontal_align_char(self): ...
    @horizontal_align_char.setter
    def horizontal_align_char(self, val) -> None: ...
    @property
    def left_junction_char(self): ...
    @left_junction_char.setter
    def left_junction_char(self, val) -> None: ...
    @property
    def none_format(self): ...
    @none_format.setter
    def none_format(self, val) -> None: ...
    @property
    def preserve_internal_border(self): ...
    @preserve_internal_border.setter
    def preserve_internal_border(self, val) -> None: ...
    @property
    def right_junction_char(self): ...
    @right_junction_char.setter
    def right_junction_char(self, val) -> None: ...
    @property
    def top_junction_char(self): ...
    @top_junction_char.setter
    def top_junction_char(self, val) -> None: ...
    @property
    def top_left_junction_char(self): ...
    @top_left_junction_char.setter
    def top_left_junction_char(self, val) -> None: ...
    @property
    def top_right_junction_char(self): ...
    @top_right_junction_char.setter
    def top_right_junction_char(self, val) -> None: ...
    @property
    def xhtml(self) -> bool: ...
    @xhtml.setter
    def xhtml(self, val: bool) -> None: ...
    @property
    def rows(self) -> list[Incomplete]: ...
    def add_autoindex(self, fieldname: str = ...): ...
    def get_latex_string(self, **kwargs) -> str: ...
    def set_style(self, style) -> None: ...
    def add_rows(self, rows) -> None: ...
    def add_row(self, row) -> None: ...
    def del_row(self, row_index) -> None: ...
    def add_column(self, fieldname, column, align: str = ..., valign: str = ...) -> None: ...
    def del_column(self, fieldname) -> None: ...
    def clear_rows(self) -> None: ...
    def clear(self) -> None: ...
    def copy(self): ...
    def get_string(self, **kwargs) -> str: ...
    def paginate(self, page_length: int = ..., line_break: str = ..., **kwargs): ...
    def get_csv_string(self, **kwargs) -> str: ...
    def get_json_string(self, **kwargs) -> str: ...
    def get_html_string(self, **kwargs) -> str: ...

def from_csv(fp, field_names: Any | None = ..., **kwargs): ...
def from_db_cursor(cursor, **kwargs): ...
def from_json(json_string, **kwargs): ...

class TableHandler(HTMLParser):
    kwargs: Any
    tables: Any
    last_row: Any
    rows: Any
    max_row_width: int
    active: Any
    last_content: str
    is_last_row_header: bool
    colspan: int
    def __init__(self, **kwargs) -> None: ...
    def handle_starttag(self, tag, attrs) -> None: ...
    def handle_endtag(self, tag) -> None: ...
    def handle_data(self, data) -> None: ...
    def generate_table(self, rows): ...
    def make_fields_unique(self, fields) -> None: ...

def from_html(html_code, **kwargs): ...
def from_html_one(html_code, **kwargs): ...



from _typeshed import StrPath, SupportsRead
from collections.abc import MutableSequence, Sequence
from typing import Any
from typing_extensions import Literal

g_token_cache: dict[str, list[tuple[str, str]]]  # undocumented
python3: Literal[True]

def unicode(x: str, y: str) -> str: ...
def render(
    template: SupportsRead[str] | str | Sequence[tuple[str, str]] = ...,
    data: dict[str, Any] = ...,
    partials_path: StrPath | None = ...,
    partials_ext: str = ...,
    partials_dict: dict[str, str] = ...,
    padding: str = ...,
    def_ldel: str | None = ...,
    def_rdel: str | None = ...,
    scopes: MutableSequence[int] | None = ...,
    warn: bool = ...,
) -> str: ...



from _typeshed import Self
from typing import ClassVar

import passlib.utils.handlers as uh
from passlib.utils.handlers import PrefixWrapper

class Pbkdf2DigestHandler(uh.HasRounds, uh.HasRawSalt, uh.HasRawChecksum, uh.GenericHandler):  # type: ignore[misc]
    checksum_chars: ClassVar[str]
    default_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    default_rounds: ClassVar[int]
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int]
    rounds_cost: ClassVar[str]
    @classmethod
    def from_string(cls: type[Self], hash: str | bytes) -> Self: ...  # type: ignore[override]

# dynamically created by create_pbkdf2_hash()
class pbkdf2_sha1(Pbkdf2DigestHandler):
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    encoded_checksum_size: ClassVar[int]

# dynamically created by create_pbkdf2_hash()
class pbkdf2_sha256(Pbkdf2DigestHandler):
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    encoded_checksum_size: ClassVar[int]

# dynamically created by create_pbkdf2_hash()
class pbkdf2_sha512(Pbkdf2DigestHandler):
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    encoded_checksum_size: ClassVar[int]

ldap_pbkdf2_sha1: PrefixWrapper
ldap_pbkdf2_sha256: PrefixWrapper
ldap_pbkdf2_sha512: PrefixWrapper

class cta_pbkdf2_sha1(uh.HasRounds, uh.HasRawSalt, uh.HasRawChecksum, uh.GenericHandler):  # type: ignore[misc]
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    default_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    default_rounds: ClassVar[int]
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int]
    rounds_cost: ClassVar[str]
    @classmethod
    def from_string(cls, hash): ...

class dlitz_pbkdf2_sha1(uh.HasRounds, uh.HasSalt, uh.GenericHandler):  # type: ignore[misc]
    name: ClassVar[str]
    ident: ClassVar[str]
    default_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    salt_chars: ClassVar[str]
    default_rounds: ClassVar[int]
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int]
    rounds_cost: ClassVar[str]
    @classmethod
    def from_string(cls, hash): ...

class atlassian_pbkdf2_sha1(uh.HasRawSalt, uh.HasRawChecksum, uh.GenericHandler):
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    @classmethod
    def from_string(cls, hash): ...

class grub_pbkdf2_sha512(uh.HasRounds, uh.HasRawSalt, uh.HasRawChecksum, uh.GenericHandler):  # type: ignore[misc]
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    default_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    default_rounds: ClassVar[int]
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int]
    rounds_cost: ClassVar[str]
    @classmethod
    def from_string(cls, hash): ...



from typing import Any
from typing_extensions import TypeAlias

# Enable when pyasn1 gets stubs:
# from pyasn1.type.univ import Enumerated, Sequence
_Enumerated: TypeAlias = Any
_Sequence: TypeAlias = Any

class PersistentSearchControl(_Sequence):
    componentType: Any

class ChangeType(_Enumerated):
    namedValues: Any

class EntryChangeNotificationControl(_Sequence):
    componentType: Any

def persistent_search_control(change_types, changes_only: bool = ..., return_ecs: bool = ..., criticality: bool = ...): ...



from typing import Match, Optional, Pattern
from typing_extensions import assert_type


def test_search(str_pat: Pattern[str], bytes_pat: Pattern[bytes]) -> None:
    assert_type(str_pat.search("x"), Optional[Match[str]])
    assert_type(bytes_pat.search(b"x"), Optional[Match[bytes]])
    assert_type(bytes_pat.search(bytearray(b"x")), Optional[Match[bytes]])



from typing import Any

from .base import GrantTypeBase as GrantTypeBase

log: Any

class RefreshTokenGrant(GrantTypeBase):
    def __init__(self, request_validator: Any | None = ..., issue_new_refresh_tokens: bool = ..., **kwargs) -> None: ...
    def create_token_response(self, request, token_handler): ...
    def validate_token_request(self, request) -> None: ...



import ast
from collections.abc import Generator
from typing import Any, ClassVar

class Plugin:
    name: ClassVar[str]
    version: ClassVar[str]
    def __init__(self, tree: ast.AST) -> None: ...
    def run(self) -> Generator[tuple[int, int, str, type[Any]], None, None]: ...



from _typeshed import ReadableBuffer, SupportsRead, SupportsWrite
from typing import Any

version: int

def dump(__value: Any, __file: SupportsWrite[bytes], __version: int = ...) -> None: ...
def load(__file: SupportsRead[bytes]) -> Any: ...
def dumps(__value: Any, __version: int = ...) -> bytes: ...
def loads(__bytes: ReadableBuffer) -> Any: ...



#!/usr/bin/env python3
"""Run mypy on the test cases for the stdlib and third-party stubs."""

from __future__ import annotations

import argparse
import os
import shutil
import subprocess
import sys
import tempfile
from itertools import product
from pathlib import Path
from typing_extensions import TypeAlias

from utils import (
    PackageInfo,
    colored,
    get_all_testcase_directories,
    get_recursive_requirements,
    print_error,
    print_success_msg,
    testcase_dir_from_package_name,
)

ReturnCode: TypeAlias = int

SUPPORTED_PLATFORMS = ["linux", "darwin", "win32"]
SUPPORTED_VERSIONS = ["3.11", "3.10", "3.9", "3.8", "3.7"]


def package_with_test_cases(package_name: str) -> PackageInfo:
    """Helper function for argument-parsing"""

    if package_name == "stdlib":
        return PackageInfo("stdlib", Path("test_cases"))
    test_case_dir = testcase_dir_from_package_name(package_name)
    if test_case_dir.is_dir():
        return PackageInfo(package_name, test_case_dir)
    raise argparse.ArgumentTypeError(f"No test cases found for {package_name!r}!")


parser = argparse.ArgumentParser(description="Script to run mypy against various test cases for typeshed's stubs")
parser.add_argument(
    "packages_to_test",
    type=package_with_test_cases,
    nargs="*",
    action="extend",
    help="Test only these packages (defaults to all typeshed stubs that have test cases)",
)
parser.add_argument(
    "--all",
    action="store_true",
    help=(
        'Run tests on all available platforms and versions (defaults to "False"). '
        "Note that this cannot be specified if --platform and/or --python-version are specified."
    ),
)
parser.add_argument(
    "--platform",
    dest="platforms_to_test",
    choices=SUPPORTED_PLATFORMS,
    nargs="*",
    action="extend",
    help=(
        "Run mypy for certain OS platforms (defaults to sys.platform). "
        "Note that this cannot be specified if --all is also specified."
    ),
)
parser.add_argument(
    "-p",
    "--python-version",
    dest="versions_to_test",
    choices=SUPPORTED_VERSIONS,
    nargs="*",
    action="extend",
    help=(
        "Run mypy for certain Python versions (defaults to sys.version_info[:2])"
        "Note that this cannot be specified if --all is also specified."
    ),
)


def test_testcase_directory(package: PackageInfo, version: str, platform: str) -> ReturnCode:
    package_name, test_case_directory = package
    is_stdlib = package_name == "stdlib"

    msg = f"Running mypy --platform {platform} --python-version {version} on the "
    msg += "standard library test cases..." if is_stdlib else f"test cases for {package_name!r}..."
    print(msg, end=" ")

    flags = [
        "--python-version",
        version,
        "--show-traceback",
        "--show-error-codes",
        "--no-error-summary",
        "--platform",
        platform,
        "--no-site-packages",
        "--strict",
    ]

    # --warn-unused-ignores doesn't work for files inside typeshed.
    # SO, to work around this, we copy the test_cases directory into a TemporaryDirectory.
    with tempfile.TemporaryDirectory() as td:
        td_path = Path(td)
        new_test_case_dir = td_path / "test_cases"
        shutil.copytree(test_case_directory, new_test_case_dir)
        env_vars = dict(os.environ)
        if is_stdlib:
            flags.extend(["--custom-typeshed-dir", str(Path(__file__).parent.parent)])
        else:
            # HACK: we want to run these test cases in an isolated environment --
            # we want mypy to see all stub packages listed in the "requires" field of METADATA.toml
            # (and all stub packages required by those stub packages, etc. etc.),
            # but none of the other stubs in typeshed.
            #
            # The best way of doing that without stopping --warn-unused-ignore from working
            # seems to be to create a "new typeshed" directory in a tempdir
            # that has only the required stubs copied over.
            new_typeshed = td_path / "typeshed"
            os.mkdir(new_typeshed)
            shutil.copytree(Path("stdlib"), new_typeshed / "stdlib")
            requirements = get_recursive_requirements(package_name)
            # mypy refuses to consider a directory a "valid typeshed directory"
            # unless there's a stubs/mypy-extensions path inside it,
            # so add that to the list of stubs to copy over to the new directory
            for requirement in requirements + ["mypy-extensions"]:
                shutil.copytree(Path("stubs", requirement), new_typeshed / "stubs" / requirement)
            env_vars["MYPYPATH"] = os.pathsep.join(map(str, new_typeshed.glob("stubs/*")))
            flags.extend(["--custom-typeshed-dir", str(td_path / "typeshed")])
        result = subprocess.run([sys.executable, "-m", "mypy", new_test_case_dir, *flags], capture_output=True, env=env_vars)

    if result.returncode:
        print_error("failure\n")
        replacements = (str(new_test_case_dir), str(test_case_directory))
        if result.stderr:
            print_error(result.stderr.decode(), fix_path=replacements)
        if result.stdout:
            print_error(result.stdout.decode(), fix_path=replacements)
    else:
        print_success_msg()
    return result.returncode


def main() -> ReturnCode:
    args = parser.parse_args()

    testcase_directories = args.packages_to_test or get_all_testcase_directories()
    if args.all:
        if args.platforms_to_test:
            raise TypeError("Cannot specify both --platform and --all")
        if args.versions_to_test:
            raise TypeError("Cannot specify both --python-version and --all")
        platforms_to_test, versions_to_test = SUPPORTED_PLATFORMS, SUPPORTED_VERSIONS
    else:
        platforms_to_test = args.platforms_to_test or [sys.platform]
        versions_to_test = args.versions_to_test or [f"3.{sys.version_info[1]}"]

    code = 0
    for platform, version, directory in product(platforms_to_test, versions_to_test, testcase_directories):
        code = max(code, test_testcase_directory(directory, version, platform))
    if code:
        print_error("\nTest completed with errors")
    else:
        print(colored("\nTest completed successfully!", "green"))

    return code


if __name__ == "__main__":
    try:
        code = main()
    except KeyboardInterrupt:
        print_error("Test aborted due to KeyboardInterrupt!")
        code = 1
    raise SystemExit(code)



from types import CodeType

__all__ = ["compile_command", "Compile", "CommandCompiler"]

def compile_command(source: str, filename: str = ..., symbol: str = ...) -> CodeType | None: ...

class Compile:
    flags: int
    def __call__(self, source: str, filename: str, symbol: str) -> CodeType: ...

class CommandCompiler:
    compiler: Compile
    def __call__(self, source: str, filename: str = ..., symbol: str = ...) -> CodeType | None: ...



from _typeshed import Incomplete, Self
from enum import Enum, Flag, IntEnum

from .syntax import Name

class DocumentState(IntEnum):
    UNINITIALIZED: int
    READY: int
    GENERATING_PAGE: int
    CLOSED: int

class SignatureFlag(IntEnum):
    SIGNATURES_EXIST: int
    APPEND_ONLY: int

class CoerciveEnum(Enum):
    @classmethod
    def coerce(cls: type[Self], value: Self | str) -> Self: ...

class CoerciveIntEnum(IntEnum):
    @classmethod
    def coerce(cls: type[Self], value: Self | str | int) -> Self: ...

class CharVPos(CoerciveEnum):
    SUP: str
    SUB: str
    NOM: str
    DENOM: str
    LINE: str

class Align(CoerciveEnum):
    C: str
    X: str
    L: str
    R: str
    J: str

class RenderStyle(CoerciveEnum):
    D: str
    F: str
    DF: str
    @property
    def operator(self) -> str: ...
    @property
    def is_draw(self) -> bool: ...
    @property
    def is_fill(self) -> bool: ...

class TextMode(CoerciveIntEnum):
    FILL: int
    STROKE: int
    FILL_STROKE: int
    INVISIBLE: int
    FILL_CLIP: int
    STROKE_CLIP: int
    FILL_STROKE_CLIP: int
    CLIP: int

class XPos(CoerciveEnum):
    LEFT: str
    RIGHT: str
    START: str
    END: str
    WCONT: str
    CENTER: str
    LMARGIN: str
    RMARGIN: str

class YPos(CoerciveEnum):
    TOP: str
    LAST: str
    NEXT: str
    TMARGIN: str
    BMARGIN: str

class PageLayout(CoerciveEnum):
    SINGLE_PAGE: Name
    ONE_COLUMN: Name
    TWO_COLUMN_LEFT: Name
    TWO_COLUMN_RIGHT: Name
    TWO_PAGE_LEFT: Name
    TWO_PAGE_RIGHT: Name

class PageMode(CoerciveEnum):
    USE_NONE: Name
    USE_OUTLINES: Name
    USE_THUMBS: Name
    FULL_SCREEN: Name
    USE_OC: Name
    USE_ATTACHMENTS: Name

class TextMarkupType(CoerciveEnum):
    HIGHLIGHT: Name
    UNDERLINE: Name
    SQUIGGLY: Name
    STRIKE_OUT: Name

class BlendMode(CoerciveEnum):
    NORMAL: Name
    MULTIPLY: Name
    SCREEN: Name
    OVERLAY: Name
    DARKEN: Name
    LIGHTEN: Name
    COLOR_DODGE: Name
    COLOR_BURN: Name
    HARD_LIGHT: Name
    SOFT_LIGHT: Name
    DIFFERENCE: Name
    EXCLUSION: Name
    HUE: Name
    SATURATION: Name
    COLOR: Name
    LUMINOSITY: Name

class AnnotationFlag(CoerciveIntEnum):
    INVISIBLE: int
    HIDDEN: int
    PRINT: int
    NO_ZOOM: int
    NO_ROTATE: int
    NO_VIEW: int
    READ_ONLY: int
    LOCKED: int
    TOGGLE_NO_VIEW: int
    LOCKED_CONTENTS: int

class AnnotationName(CoerciveEnum):
    NOTE: Name
    COMMENT: Name
    HELP: Name
    PARAGRAPH: Name
    NEW_PARAGRAPH: Name
    INSERT: Name

class FileAttachmentAnnotationName(CoerciveEnum):
    PUSH_PIN: Name
    GRAPH_PUSH_PIN: Name
    PAPERCLIP_TAG: Name

class IntersectionRule(CoerciveEnum):
    NONZERO: str
    EVENODD: str

class PathPaintRule(CoerciveEnum):
    STROKE: str
    FILL_NONZERO: str
    FILL_EVENODD: str
    STROKE_FILL_NONZERO: str
    STROKE_FILL_EVENODD: str
    DONT_PAINT: str
    AUTO: str

class ClippingPathIntersectionRule(CoerciveEnum):
    NONZERO: str
    EVENODD: str

class StrokeCapStyle(CoerciveIntEnum):
    BUTT: int
    ROUND: int
    SQUARE: int

class StrokeJoinStyle(CoerciveIntEnum):
    MITER: int
    ROUND: int
    BEVEL: int

class PDFStyleKeys(Enum):
    FILL_ALPHA: Name
    BLEND_MODE: Name
    STROKE_ALPHA: Name
    STROKE_ADJUSTMENT: Name
    STROKE_WIDTH: Name
    STROKE_CAP_STYLE: Name
    STROKE_JOIN_STYLE: Name
    STROKE_MITER_LIMIT: Name
    STROKE_DASH_PATTERN: Name

class Corner(CoerciveEnum):
    TOP_RIGHT: str
    TOP_LEFT: str
    BOTTOM_RIGHT: str
    BOTTOM_LEFT: str

class FontDescriptorFlags(Flag):
    FIXED_PITCH: int
    SYMBOLIC: int
    ITALIC: int
    FORCE_BOLD: int

__pdoc__: Incomplete



from typing import Any

vmlns: str
officens: str
excelns: str

class ShapeWriter:
    vml: Any
    vml_path: Any
    comments: Any
    def __init__(self, comments) -> None: ...
    def add_comment_shapetype(self, root) -> None: ...
    def add_comment_shape(self, root, idx, coord, height, width) -> None: ...
    def write(self, root): ...



from braintree.exceptions.authentication_error import AuthenticationError as AuthenticationError
from braintree.exceptions.authorization_error import AuthorizationError as AuthorizationError
from braintree.exceptions.configuration_error import ConfigurationError as ConfigurationError
from braintree.exceptions.gateway_timeout_error import GatewayTimeoutError as GatewayTimeoutError
from braintree.exceptions.invalid_challenge_error import InvalidChallengeError as InvalidChallengeError
from braintree.exceptions.invalid_signature_error import InvalidSignatureError as InvalidSignatureError
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.exceptions.request_timeout_error import RequestTimeoutError as RequestTimeoutError
from braintree.exceptions.server_error import ServerError as ServerError
from braintree.exceptions.service_unavailable_error import ServiceUnavailableError as ServiceUnavailableError
from braintree.exceptions.too_many_requests_error import TooManyRequestsError as TooManyRequestsError
from braintree.exceptions.unexpected_error import UnexpectedError as UnexpectedError
from braintree.exceptions.upgrade_required_error import UpgradeRequiredError as UpgradeRequiredError



from typing import Any

from .base import Client as Client

class WebApplicationClient(Client):
    grant_type: str
    code: Any
    def __init__(self, client_id, code: Any | None = ..., **kwargs) -> None: ...
    def prepare_request_uri(  # type: ignore[override]
        self,
        uri,
        redirect_uri: Any | None = ...,
        scope: Any | None = ...,
        state: Any | None = ...,
        code_challenge: str | None = ...,
        code_challenge_method: str | None = ...,
        **kwargs,
    ): ...
    def prepare_request_body(  # type: ignore[override]
        self,
        code: Any | None = ...,
        redirect_uri: Any | None = ...,
        body: str = ...,
        include_client_id: bool = ...,
        code_verifier: str | None = ...,
        **kwargs,
    ): ...
    def parse_request_uri_response(self, uri, state: Any | None = ...): ...  # type: ignore[override]



from typing import Any

DEFAULT_ROW_HEIGHT: float
BASE_COL_WIDTH: int
DEFAULT_COLUMN_WIDTH: Any
DEFAULT_LEFT_MARGIN: float
DEFAULT_TOP_MARGIN: float
DEFAULT_HEADER: float

def inch_to_dxa(value): ...
def dxa_to_inch(value): ...
def dxa_to_cm(value): ...
def cm_to_dxa(value): ...
def pixels_to_EMU(value): ...
def EMU_to_pixels(value): ...
def cm_to_EMU(value): ...
def EMU_to_cm(value): ...
def inch_to_EMU(value): ...
def EMU_to_inch(value): ...
def pixels_to_points(value, dpi: int = ...): ...
def points_to_pixels(value, dpi: int = ...): ...
def degrees_to_angle(value): ...
def angle_to_degrees(value): ...
def short_color(color): ...



from typing import Any, Generic, TypeVar

from ..orm.query import Query
from ..orm.session import Session

_T = TypeVar("_T")

class ShardedQuery(Query[_T], Generic[_T]):
    id_chooser: Any
    query_chooser: Any
    execute_chooser: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def set_shard(self, shard_id): ...

class ShardedSession(Session):
    shard_chooser: Any
    id_chooser: Any
    execute_chooser: Any
    query_chooser: Any
    def __init__(
        self, shard_chooser, id_chooser, execute_chooser: Any | None = ..., shards: Any | None = ..., query_cls=..., **kwargs
    ): ...
    def connection_callable(self, mapper: Any | None = ..., instance: Any | None = ..., shard_id: Any | None = ..., **kwargs): ...
    def get_bind(self, mapper: Any | None = ..., shard_id: Any | None = ..., instance: Any | None = ..., clause: Any | None = ..., **kw): ...  # type: ignore[override]
    def bind_shard(self, shard_id, bind) -> None: ...



import html
from typing import Any

HTMLunescape = html.unescape
ENTITY: str
TAGNAME: str
ATTRIBUTENAME: str
UNQUOTEDVALUE: str
SINGLEQUOTEDVALUE: str
DOUBLEQUOTEDVALUE: str
ATTRIBUTEVALUE: Any
ATTRIBUTEVALUESPEC: Any
ATTRIBUTE: Any
OPENTAG: Any
CLOSETAG: Any
HTMLCOMMENT: str
PROCESSINGINSTRUCTION: str
DECLARATION: Any
CDATA: str
HTMLTAG: Any
reHtmlTag: Any
reBackslashOrAmp: Any
ESCAPABLE: str
reEntityOrEscapedChar: Any
XMLSPECIAL: str
reXmlSpecial: Any

def unescape_char(s): ...
def unescape_string(s): ...
def normalize_uri(uri): ...

UNSAFE_MAP: Any

def replace_unsafe_char(s): ...
def escape_xml(s): ...



from .plugin import Error as Error, Plugin as Plugin, Visitor as Visitor
from .utils import (
    assert_error as assert_error,
    assert_not_error as assert_not_error,
    check_equivalent_nodes as check_equivalent_nodes,
    is_false as is_false,
    is_none as is_none,
    is_true as is_true,
)



import logging
from typing import Any

# This pattern comes from the logging docs, and should therefore pass a type checker
# See https://docs.python.org/3/library/logging.html#logrecord-objects

old_factory = logging.getLogRecordFactory()


def record_factory(*args: Any, **kwargs: Any) -> logging.LogRecord:
    record = old_factory(*args, **kwargs)
    record.custom_attribute = 0xDECAFBAD
    return record


logging.setLogRecordFactory(record_factory)



from typing import Any

from braintree.dispute import Dispute as Dispute
from braintree.dispute_details import DisputeEvidence as DisputeEvidence
from braintree.error_result import ErrorResult as ErrorResult
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.paginated_collection import PaginatedCollection as PaginatedCollection
from braintree.paginated_result import PaginatedResult as PaginatedResult
from braintree.resource_collection import ResourceCollection as ResourceCollection
from braintree.successful_result import SuccessfulResult as SuccessfulResult

class DisputeGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def accept(self, dispute_id): ...
    def add_file_evidence(self, dispute_id, document_upload_id_or_request): ...
    def add_text_evidence(self, dispute_id, content_or_request): ...
    def finalize(self, dispute_id): ...
    def find(self, dispute_id): ...
    def remove_evidence(self, dispute_id, evidence_id): ...
    search_criteria: Any
    def search(self, *query): ...



# Can't generate with stubgen because:
# "ImportError: This must be an MFC application - try 'import win32ui' first"
APPCLASS_MONITOR: int
APPCLASS_STANDARD: int
APPCMD_CLIENTONLY: int
APPCMD_FILTERINITS: int
CBF_FAIL_ADVISES: int
CBF_FAIL_ALLSVRXACTIONS: int
CBF_FAIL_CONNECTIONS: int
CBF_FAIL_EXECUTES: int
CBF_FAIL_POKES: int
CBF_FAIL_REQUESTS: int
CBF_FAIL_SELFCONNECTIONS: int
CBF_SKIP_ALLNOTIFICATIONS: int
CBF_SKIP_CONNECT_CONFIRMS: int
CBF_SKIP_DISCONNECTS: int
CBF_SKIP_REGISTRATIONS: int

def CreateConversation(*args, **kwargs): ...  # incomplete
def CreateServer(*args, **kwargs): ...  # incomplete
def CreateServerSystemTopic(*args, **kwargs): ...  # incomplete
def CreateStringItem(*args, **kwargs): ...  # incomplete
def CreateTopic(*args, **kwargs): ...  # incomplete

MF_CALLBACKS: int
MF_CONV: int
MF_ERRORS: int
MF_HSZ_INFO: int
MF_LINKS: int
MF_POSTMSGS: int
MF_SENDMSGS: int

class error(Exception): ...



from collections.abc import Generator
from typing import Any

class WorkbookParser:
    archive: Any
    workbook_part_name: Any
    wb: Any
    keep_links: Any
    sheets: Any
    def __init__(self, archive, workbook_part_name, keep_links: bool = ...) -> None: ...
    @property
    def rels(self): ...
    caches: Any
    def parse(self) -> None: ...
    def find_sheets(self) -> Generator[Any, None, None]: ...
    def assign_names(self) -> None: ...
    @property
    def pivot_caches(self): ...



from _typeshed import Incomplete
from collections.abc import Sequence
from typing_extensions import Literal

DEFAULT_STRATEGY: Literal[0]
FILTERED: Literal[1]
HUFFMAN_ONLY: Literal[2]
RLE: Literal[3]
FIXED: Literal[4]

class _Path:
    def __getattr__(self, item: str) -> Incomplete: ...

def path(__x: Sequence[tuple[float, float]] | Sequence[float]) -> _Path: ...
def __getattr__(__name: str) -> Incomplete: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class Related(Serialisable):  # type: ignore[misc]
    id: Any
    def __init__(self, id: Any | None = ...) -> None: ...
    def to_tree(self, tagname, idx: Any | None = ...): ...  # type: ignore[override]



from collections.abc import Callable

from .client import Client, MQTTMessage, _UserData
from .publish import _TLS, _Auth, _Msg, _Proxy

def callback(
    callback: Callable[[Client, _UserData, MQTTMessage], None],
    topics: list[str],
    qos: int = ...,
    userdata: _UserData | None = ...,
    hostname: str = ...,
    port: int = ...,
    client_id: str = ...,
    keepalive: int = ...,
    will: _Msg | None = ...,
    auth: _Auth | None = ...,
    tls: _TLS | None = ...,
    protocol: int = ...,
    transport: str = ...,
    clean_session: bool = ...,
    proxy_args: _Proxy | None = ...,
) -> None: ...
def simple(
    topics: str | list[str],
    qos: int = ...,
    msg_count: int = ...,
    retained: bool = ...,
    hostname: str = ...,
    port: int = ...,
    client_id: str = ...,
    keepalive: int = ...,
    will: _Msg | None = ...,
    auth: _Auth | None = ...,
    tls: _TLS | None = ...,
    protocol: int = ...,
    transport: str = ...,
    clean_session: bool = ...,
    proxy_args: _Proxy | None = ...,
) -> list[MQTTMessage] | MQTTMessage: ...



from collections.abc import Iterable

from . import patch as patch
from .exceptions import HunkApplyException as HunkApplyException, SubprocessException as SubprocessException
from .snippets import remove as remove, which as which

def apply_patch(diffs: patch.diffobj | Iterable[patch.diffobj]) -> None: ...
def apply_diff(diff: patch.diffobj, text: str | Iterable[str], reverse: bool = ..., use_patch: bool = ...) -> list[str]: ...



from typing import Any

from braintree.payment_method_parser import parse_payment_method as parse_payment_method
from braintree.resource import Resource as Resource

class RevokedPaymentMethodMetadata(Resource):
    revoked_payment_method: Any
    customer_id: Any
    token: Any
    def __init__(self, gateway, attributes) -> None: ...



from typing import Any

class TSInfo:
    rules: list[Any]
    labels: list[Any]
    sourceKey: Any | None
    chunk_count: Any | None
    memory_usage: Any | None
    total_samples: Any | None
    retention_msecs: Any | None
    last_time_stamp: Any | None
    first_time_stamp: Any | None

    max_samples_per_chunk: Any | None
    chunk_size: Any | None
    duplicate_policy: Any | None
    def __init__(self, args) -> None: ...



from datetime import datetime as datetime

from aws_xray_sdk.core.context import Context as Context
from aws_xray_sdk.core.models.dummy_entities import DummySegment as DummySegment
from aws_xray_sdk.core.utils.compat import PY2 as PY2

from .sampling_rule import SamplingRule as SamplingRule

class ServiceConnector:
    def __init__(self) -> None: ...
    def fetch_sampling_rules(self): ...
    def fetch_sampling_target(self, rules): ...
    def setup_xray_client(self, ip, port, client) -> None: ...
    @property
    def context(self): ...
    @context.setter
    def context(self, v) -> None: ...



from typing import Any

class Node:
    id: Any
    alias: Any
    label: Any
    labels: Any
    properties: Any
    def __init__(
        self,
        node_id: Any | None = ...,
        alias: Any | None = ...,
        label: str | list[str] | None = ...,
        properties: Any | None = ...,
    ) -> None: ...
    def to_string(self): ...
    def __eq__(self, rhs): ...



from _typeshed import Self
from collections.abc import Callable
from hashlib import _Hash
from typing import Any

from .base import Key

# Enable when we can use stubs from installed dependencies:
# from ecdsa.curves import Curve
class ECDSAECKey(Key):
    SHA256: Callable[[bytes], _Hash]
    SHA384: Callable[[bytes], _Hash]
    SHA512: Callable[[bytes], _Hash]
    CURVE_MAP: Any
    CURVE_NAMES: Any
    hash_alg: Any
    curve: Any
    prepared_key: Any
    def __init__(self, key, algorithm) -> None: ...
    def sign(self, msg): ...
    def verify(self, msg, sig): ...
    def is_public(self) -> bool: ...
    def public_key(self: Self) -> Self: ...
    def to_pem(self): ...
    def to_dict(self) -> dict[str, Any]: ...



from collections.abc import Mapping, Sequence
from io import BytesIO
from re import Pattern
from typing import Any

from waitress.adjustments import Adjustments
from waitress.receiver import ChunkedReceiver, FixedStreamReceiver
from waitress.utilities import Error

class ParsingError(Exception): ...
class TransferEncodingNotImplemented(Exception): ...

class HTTPRequestParser:
    completed: bool = ...
    empty: bool = ...
    expect_continue: bool = ...
    headers_finished: bool = ...
    header_plus: bytes = ...
    chunked: bool = ...
    content_length: int = ...
    header_bytes_received: int = ...
    body_bytes_received: int = ...
    body_rcv: ChunkedReceiver | FixedStreamReceiver | None = ...
    version: str = ...
    error: Error | None = ...
    connection_close: bool = ...
    headers: Mapping[str, str] = ...
    adj: Adjustments = ...
    def __init__(self, adj: Adjustments) -> None: ...
    def received(self, data: bytes) -> int: ...
    first_line: str = ...
    command: bytes = ...
    url_scheme: str = ...
    def parse_header(self, header_plus: bytes) -> None: ...
    def get_body_stream(self) -> BytesIO: ...
    def close(self) -> None: ...

def split_uri(uri: bytes) -> tuple[str, str, bytes, str, str]: ...
def get_header_lines(header: bytes) -> Sequence[bytes]: ...

first_line_re: Pattern[Any]

def crack_first_line(line: str) -> tuple[bytes, bytes, bytes]: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class ObjectAnchor(Serialisable):
    tagname: str
    to: Any
    moveWithCells: Any
    sizeWithCells: Any
    z_order: Any
    def __init__(
        self,
        _from: Any | None = ...,
        to: Any | None = ...,
        moveWithCells: bool = ...,
        sizeWithCells: bool = ...,
        z_order: Any | None = ...,
    ) -> None: ...

class ObjectPr(Serialisable):
    tagname: str
    anchor: Any
    locked: Any
    defaultSize: Any
    disabled: Any
    uiObject: Any
    autoFill: Any
    autoLine: Any
    autoPict: Any
    macro: Any
    altText: Any
    dde: Any
    __elements__: Any
    def __init__(
        self,
        anchor: Any | None = ...,
        locked: bool = ...,
        defaultSize: bool = ...,
        _print: bool = ...,
        disabled: bool = ...,
        uiObject: bool = ...,
        autoFill: bool = ...,
        autoLine: bool = ...,
        autoPict: bool = ...,
        macro: Any | None = ...,
        altText: Any | None = ...,
        dde: bool = ...,
    ) -> None: ...

class OleObject(Serialisable):
    tagname: str
    objectPr: Any
    progId: Any
    dvAspect: Any
    link: Any
    oleUpdate: Any
    autoLoad: Any
    shapeId: Any
    __elements__: Any
    def __init__(
        self,
        objectPr: Any | None = ...,
        progId: Any | None = ...,
        dvAspect: str = ...,
        link: Any | None = ...,
        oleUpdate: Any | None = ...,
        autoLoad: bool = ...,
        shapeId: Any | None = ...,
    ) -> None: ...

class OleObjects(Serialisable):
    tagname: str
    oleObject: Any
    __elements__: Any
    def __init__(self, oleObject=...) -> None: ...



from abc import abstractmethod
from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class ChartLines(Serialisable):
    tagname: str
    spPr: Any
    graphicalProperties: Any
    def __init__(self, spPr: Any | None = ...) -> None: ...

class Scaling(Serialisable):
    tagname: str
    logBase: Any
    orientation: Any
    max: Any
    min: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        logBase: Any | None = ...,
        orientation: str = ...,
        max: Any | None = ...,
        min: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class _BaseAxis(Serialisable):
    axId: Any
    scaling: Any
    delete: Any
    axPos: Any
    majorGridlines: Any
    minorGridlines: Any
    title: Any
    numFmt: Any
    number_format: Any
    majorTickMark: Any
    minorTickMark: Any
    tickLblPos: Any
    spPr: Any
    graphicalProperties: Any
    txPr: Any
    textProperties: Any
    crossAx: Any
    crosses: Any
    crossesAt: Any
    __elements__: Any
    def __init__(
        self,
        axId: Any | None = ...,
        scaling: Any | None = ...,
        delete: Any | None = ...,
        axPos: str = ...,
        majorGridlines: Any | None = ...,
        minorGridlines: Any | None = ...,
        title: Any | None = ...,
        numFmt: Any | None = ...,
        majorTickMark: Any | None = ...,
        minorTickMark: Any | None = ...,
        tickLblPos: Any | None = ...,
        spPr: Any | None = ...,
        txPr: Any | None = ...,
        crossAx: Any | None = ...,
        crosses: Any | None = ...,
        crossesAt: Any | None = ...,
    ) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class DisplayUnitsLabel(Serialisable):
    tagname: str
    layout: Any
    tx: Any
    text: Any
    spPr: Any
    graphicalProperties: Any
    txPr: Any
    textPropertes: Any
    __elements__: Any
    def __init__(
        self, layout: Any | None = ..., tx: Any | None = ..., spPr: Any | None = ..., txPr: Any | None = ...
    ) -> None: ...

class DisplayUnitsLabelList(Serialisable):
    tagname: str
    custUnit: Any
    builtInUnit: Any
    dispUnitsLbl: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self, custUnit: Any | None = ..., builtInUnit: Any | None = ..., dispUnitsLbl: Any | None = ..., extLst: Any | None = ...
    ) -> None: ...

class NumericAxis(_BaseAxis):
    tagname: str
    axId: Any
    scaling: Any
    delete: Any
    axPos: Any
    majorGridlines: Any
    minorGridlines: Any
    title: Any
    numFmt: Any
    majorTickMark: Any
    minorTickMark: Any
    tickLblPos: Any
    spPr: Any
    txPr: Any
    crossAx: Any
    crosses: Any
    crossesAt: Any
    crossBetween: Any
    majorUnit: Any
    minorUnit: Any
    dispUnits: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        crossBetween: Any | None = ...,
        majorUnit: Any | None = ...,
        minorUnit: Any | None = ...,
        dispUnits: Any | None = ...,
        extLst: Any | None = ...,
        **kw,
    ) -> None: ...
    @classmethod
    def from_tree(cls, node): ...

class TextAxis(_BaseAxis):
    tagname: str
    axId: Any
    scaling: Any
    delete: Any
    axPos: Any
    majorGridlines: Any
    minorGridlines: Any
    title: Any
    numFmt: Any
    majorTickMark: Any
    minorTickMark: Any
    tickLblPos: Any
    spPr: Any
    txPr: Any
    crossAx: Any
    crosses: Any
    crossesAt: Any
    auto: Any
    lblAlgn: Any
    lblOffset: Any
    tickLblSkip: Any
    tickMarkSkip: Any
    noMultiLvlLbl: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        auto: Any | None = ...,
        lblAlgn: Any | None = ...,
        lblOffset: int = ...,
        tickLblSkip: Any | None = ...,
        tickMarkSkip: Any | None = ...,
        noMultiLvlLbl: Any | None = ...,
        extLst: Any | None = ...,
        **kw,
    ) -> None: ...

class DateAxis(TextAxis):
    tagname: str
    axId: Any
    scaling: Any
    delete: Any
    axPos: Any
    majorGridlines: Any
    minorGridlines: Any
    title: Any
    numFmt: Any
    majorTickMark: Any
    minorTickMark: Any
    tickLblPos: Any
    spPr: Any
    txPr: Any
    crossAx: Any
    crosses: Any
    crossesAt: Any
    auto: Any
    lblOffset: Any
    baseTimeUnit: Any
    majorUnit: Any
    majorTimeUnit: Any
    minorUnit: Any
    minorTimeUnit: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        auto: Any | None = ...,
        lblOffset: Any | None = ...,
        baseTimeUnit: Any | None = ...,
        majorUnit: Any | None = ...,
        majorTimeUnit: Any | None = ...,
        minorUnit: Any | None = ...,
        minorTimeUnit: Any | None = ...,
        extLst: Any | None = ...,
        **kw,
    ) -> None: ...

class SeriesAxis(_BaseAxis):
    tagname: str
    axId: Any
    scaling: Any
    delete: Any
    axPos: Any
    majorGridlines: Any
    minorGridlines: Any
    title: Any
    numFmt: Any
    majorTickMark: Any
    minorTickMark: Any
    tickLblPos: Any
    spPr: Any
    txPr: Any
    crossAx: Any
    crosses: Any
    crossesAt: Any
    tickLblSkip: Any
    tickMarkSkip: Any
    extLst: Any
    __elements__: Any
    def __init__(self, tickLblSkip: Any | None = ..., tickMarkSkip: Any | None = ..., extLst: Any | None = ..., **kw) -> None: ...
